#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATDef {} {
    global cat

    set cat(id) 0
    set cat(cats) {}
    set cat(server) cds
    set cat(system) wcs
    set cat(sky) fk5
    set cat(skyformat) sexagesimal
    set cat(rformat) arcmin
    set cat(vot) 1

    set cat(width) 15
    set cat(height) 15
    set cat(max) 5000
    set cat(allrows) 1
    set cat(allcols) 0
    set cat(show) 1
    set cat(edit) 0
    set cat(panto) 1

    set cat(minrows) 20
    set cat(mincols) 10

    set cat(source) {}
    set cat(words) {}
    set cat(wave) {}
    set cat(mission) {}
    set cat(astro) {}

    set cat(sym,shape) {circle point}
    set cat(sym,color) green
    set cat(sym,units) physical

    set cat(def) { \
		       {- {Database} db} \
		       {{NED} catned ned ned} \
		       {{SIMBAD} catsimbad simbad simbad} \
		       {{DENIS} catdenis cds {B/denis}} \
		       {- {Optical} opt} \
		       {{ASCC-2.5} catascss cds {I/280A/ascc01}} \
		       {{Carlsberg Meridian 14} catcmc cds {I/304}}\
		       {{GSC 1.2} catgsc1 cds {I/254/out}} \
		       {{GSC 2.2} catgsc2 cds {I/271/out}} \
		       {{GSC 2.3} catgsc3 cds {I/305/out}} \
		       {{AC 2000.2} catac cds {I/275/ac2002}} \
		       {{NOMAD} catnomad cds {I/297/out}} \
		       {{PPMX} catppmx cds {I/312}} \
		       {{SAO J2000} catsao cds {I/131A/sao}} \
		       {{SDSS Release 5} catsdss5 cds {II/276}} \
		       {{SDSS Release 6} catsdss6 cds {II/282}} \
		       {{Tycho-2} cattycho cds {I/259/tyc2}} \
		       {{USNO-A2.0} catua2 cds {I/252/out}} \
		       {{USNO-B1.0} catub1 cds {I/284/out}} \
		       {{USNO UCAC2} catucac2 cds {I/289/out}} \
		       {- {Infrared} ir} \
		       {{2MASS Point Sources} cat2mass cds {II/246/out}}\
		       {{IRAS Point Sources} catiras cds {II/125}}\
		       {- {High Energy} hea} \
		       {{Chandra Source} catcsc cxc {Current Release}}
		       {{2XMMi Source} catxmm cds {IX/40/xmm2is}} \
		       {{Second ROSAT PSPC} catrosat cds {IX/30}} \
		       {- {Radio} radio} \
		       {{FIRST Survey} catfirst cds {VIII/71/first}} \
		       {{NVSS} catnvss cds {VIII/65/nvss}} \
		       {- {Observation Logs} log} \
		       {{Chandra Archive} catchandralog cds {B/chandra/chandra}} \
		       {{CFHT Exposures} catcfhtlog cds {B/cfht/chfht}} \
		       {{ESO Science Archive} catesolog cds {B/eso/safcat}} \
		       {{HST Archive} cathstlog cds {B/hst/hstlog}} \
		       {{XMM Observation} catxmmlog cds {B/xmm/xmmlog}} \
		   }

}

# Load via HTTP

proc CATGetURL {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATGetURL $varname $var(url)?$var(query)"
    }

    ARStatus $varname [msgcat::mc {Loading}]

    global http

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    CATGetURLFinish $varname $var(token)
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -command \
					 [list CATGetURLFinish $varname] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    }
}

proc CATGetURLIncr {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATGetURLIncr $varname $var(url)?$var(query)"
    }

    ARStatus $varname [msgcat::mc {Loading}]

    global http

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(proc,reader) $var(catdb)] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    CATGetURLFinish $varname $var(token)
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(proc,reader) $var(catdb)] \
					 -command \
					 [list CATGetURLFinish $varname] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    }
}

proc CATGetURLFinish {varname token} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATGetURLFinish $varname"
    }

    if {!($var(active))} {
	ARCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	{} -
	200 -
	203 -
	404 -
	503 {
	    if [info exist ${varname}(proc,parser)] {
		eval [list $var(proc,parser) $var(catdb) $token]
	    }

	    ARDone $varname
	    eval $var(proc,done) $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    global debug
		    if {$debug(tcl,cat)} {
			puts stderr "CATGetURLFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    eval $var(proc,load) $varname
		}
	    }
	}

	default {ARError $varname "[msgcat::mc {Error code was returned}] $code"}
    }
}

proc CATLoad {varname} {
    upvar #0 $varname var
    global $varname

    # clear previous db
    global $var(catdb)
    if [info exists $var(catdb)] {
	unset $var(catdb)
    }

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATLoad $varname $var(url)?$var(query)"
    }

    set var(proc,done) CATLoadDone
    set var(proc,load) CATLoad
    CATGetURL $varname
    return
}

proc CATLoadIncr {varname} {
    upvar #0 $varname var
    global $varname

    # clear previous db
    global $var(catdb)
    if [info exists $var(catdb)] {
	unset $var(catdb)
    }

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATLoadIncr $varname $var(url)?$var(query)"
    }

    set var(proc,done) CATLoadDone
    set var(proc,load) CATLoadIncr
    CATGetURLIncr $varname
    return
}

proc CATLoadDone {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATLoadDone $varname"
    }

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

# Load via File

proc CATLoadSBFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn starbase_read
    }
}

proc CATLoadVOTFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catvotfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn CATVOTRead
    }
}

proc CATLoadTSVFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog cattsvfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn CATTSVRead
    }
}

proc CATLoadFn {varname fn reader} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATLoadFn $varname $fn $reader"
    }

    ARStatus $varname [msgcat::mc {Loading Catalog}]

    CATOff $varname
    CATSet $varname {} {} $fn

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(width) {}
    set var(height) {}

    if {[file exists $fn]} {
	$reader $var(catdb) $fn
    } else {
	Error "[msgcat::mc {Unable to open file}] $fn"
	return
    }

    ARDone $varname

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

# Save via File

proc CATSaveSBFile {varname} {
    set fn [SaveFileDialog catfbox]
    CATSaveFn $varname $fn starbase_write
}

proc CATSaveVOTFile {varname} {
    set fn [SaveFileDialog catvotfbox]
    CATSaveFn $varname $fn CATVOTWrite
}

proc CATSaveTSVFile {varname} {
    set fn [SaveFileDialog cattsvfbox]
    CATSaveFn $varname $fn CATTSVWrite
}

proc CATSaveFn {varname fn writer} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if {$fn == {}} {
	return
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname [msgcat::mc {Writing Catalog}]
    $writer $var(tbldb) $fn
    ARDone $varname
}

# Other procedures

proc CATOff {varname} {
    upvar #0 $varname var
    global $varname
    global current

    global $var(catdb)
    if [info exists $var(catdb)] {
	unset $var(catdb)
    }
    global $var(tbldb)
    if [info exists $var(tbldb)] {
	unset $var(tbldb)
    }
    set db $var(tbldb)
    set ${db}(Nrows) {}

    $var(tbl) selection clear all

    $var(frame) marker catalog $varname delete

    CATSortMenu $varname
    CATRADECMenu $varname
    set var(filter) {}
    set var(sort) {}
    set var(ra) {}
    set var(dec) {}

    set var(blink) 0
}

proc CATTable {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATTable $varname"
    }

    if {![CATValidDB $var(catdb)]} {
	Error [msgcat::mc {No catalog data loaded}]
	return
    }

    # delete any previous tbldb
    set db ${varname}tbldb
    global $db
    if [info exists $db] {
	unset $db
    }

    # clear the selection
    $var(tbl) selection clear all

    # clear regions
    $var(frame) marker catalog $varname delete

    if {$var(filter) == {} && $var(sort) == {}} {
	set var(tbldb) $var(catdb)
    } else {
	set var(tbldb) ${varname}tbldb
	global $var(tbldb)
	if {![CATFltSort $varname]} {
	    Error "[msgcat::mc {Unable to evaluate filter}] $var(filter)"
	    if [info exists $var(tbldb)] {
		unset $var(tbldb)
	    }
	    set var(tbldb) $var(catdb)
	}
    }

    global $var(tbldb)
    $var(tbl) configure -variable $var(tbldb)
    $var(found) configure -textvariable ${var(tbldb)}(Nrows)

#    starbase_writefp $var(catdb) stdout
#    starbase_writefp $var(tbldb) stdout

    if {[starbase_nrows $var(tbldb)] == 0} {
	ARStatus $varname [msgcat::mc {No Items Found}]
	return
    }

    set nc [starbase_ncols $var(tbldb)]
    if { $nc > $cat(mincols)} {
	$var(tbl) configure -cols $nc
    } else {
	$var(tbl) configure -cols $cat(mincols)
    }

    # add header
    set nr [expr [starbase_nrows $var(tbldb)]+1]
    if {$nr > $cat(minrows)} {
	$var(tbl) configure -rows $nr
    } else {
	$var(tbl) configure -rows $cat(minrows)
    }

    CATPlot $varname

    set nr [starbase_nrows $var(tbldb)]
    if {$nr >= $var(max) && !$var(allrows)} {
	ARStatus $varname "$nr [msgcat::mc {rows of data have been downloaded. More may be available. You may wish to adjust the maximum allowed}]"
    }
}

proc CATPlot {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATPlot $varname"
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname [msgcat::mc {Plotting Regions}]

    if {[$var(frame) has wcs equatorial wcs]} {
	# delete any previous 
	$var(frame) marker catalog $varname delete

	if {$var(show)} {
	    global reg
	    set reg {}
	    CATReg $varname {} 1 reg

	    if {$reg != {}} {
		if [catch {$var(frame) marker catalog command ds9 var reg}] {
		    ARError $varname "[msgcat::mc {Internal Parse Error}]"
		    return
		}
	    }
	}
    }

    ARStatus $varname {Done}
}

proc CATPlotRegions {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATPlotRegions $varname"
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname [msgcat::mc {Generating Regions}]

    if {[$var(frame) has wcs equatorial wcs]} {
	global reg
	set reg {}
	CATReg $varname {} 0 reg
	if {$reg != {}} {
	    if [catch {$var(frame) marker command ds9 var reg}] {
		ARError $varname "[msgcat::mc {Internal Parse Error}]"
		return
	    }
	}
    }

    ARStatus $varname {Done}
}

proc CATPlotUpdate {varname row} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATPlotUpdate $varname $row"
    }

    set id [$var(frame) get marker catalog "\{${varname}.${row}\}" id]
    set sel [$var(frame) get marker catalog $id select]
    set hh [$var(frame) get marker catalog $id highlite]

    $var(frame) marker catalog "\{${varname}.${row}\}" delete
    global reg
    set reg {}
    CATReg $varname $row 1 reg
    if {$reg != {}} {
	$var(frame) marker catalog command ds9 var reg

	set id [$var(frame) get marker catalog "\{${varname}.${row}\}" id]
	if {$sel} {
	    $var(frame) marker catalog "\{${varname}.${row}\}" select
	}
	if {$hh} {
	    $var(frame) marker catalog "\{${varname}.${row}\}" highlite
	}
    }
    unset reg
}

proc CATSet {varname format catalog title} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATSet $varname :$format:$catalog:$title:"
    }

    set var(format) $format
    set var(catalog) $catalog
    set var(title) $title
    set var(filter) {}
    set var(ra) {}
    set var(dec) {}
    set var(sort) {}
    set var(sort,dir) "-increasing"
}

proc CATValidDB {varname} {
    upvar #0 $varname var
    global $varname

    if {[info exists var(Nrows)] && 
	[info exists var(Ncols)] &&
	[info exists var(HLines)] &&
	[info exists var(Header)]} {
	return 1
    } else {
	return 0
    }
}

proc CATCatalogMenu {} {
    global cat
    global ds9

    set mm "$ds9(mb).analysis.cat"
    set nn {}

    foreach ff $cat(def) {
	set ll [lindex $ff 0]
	set ww [lindex $ff 1]
	set ss [lindex $ff 2]
	set cc [lindex $ff 3]

	if {$ll != {-}} {
	    $mm.$nn add command -label $ll \
		-command [list CATDialog $ww $ss $cc $ll apply]
	} else {
	    set nn "$ss"
	    menu $mm.$nn
	    $mm add cascade -label $ww -menu $mm.$nn
	}
    }
}

proc CATServerMenu {varname} {
    upvar #0 $varname var
    global $varname

    global cat

    $var(mb) add cascade -label [msgcat::mc {Catalog Server}] \
	-menu $var(mb).server
    menu $var(mb).server
    $var(mb).server add radiobutton -label {CDS, Strasbourg France} \
	-variable cat(server) -value cds
    $var(mb).server add radiobutton -label {CFA, Boston USA} \
	-variable cat(server) -value sao
    $var(mb).server add radiobutton -label {CADC, Canada} \
	-variable cat(server) -value cadc
    $var(mb).server add radiobutton -label {ADAC, Tokyo Japan} \
	-variable cat(server) -value adac
    $var(mb).server add radiobutton -label {IUCAA, Pune India} \
	-variable cat(server) -value iucaa
    $var(mb).server add radiobutton -label {INASAN, Russia} \
	-variable cat(server) -value inasan
    $var(mb).server add radiobutton -label {BEJING, China} \
	-variable cat(server) -value bejing
    $var(mb).server add radiobutton -label {CAMBRIDGE, UK} \
	-variable cat(server) -value cambridge
    $var(mb).server add radiobutton -label {UKIRT, Hawaii USA} \
	-variable cat(server) -value ukirt
}

proc CATSortMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global ds9

    set m $var(top).param.f.sort.m
    catch {destroy $m}

    menu $m -tearoff 0
    $m add command -label {} -command "CATSortCmd $varname {}"
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATSortCmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }
}

proc CATSortCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(sort) $val
    CATTable $varname
}

proc CATRADECMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global ds9

    set m $var(top).param.f.ra.m
    catch {destroy $m}

    menu $m -tearoff 0
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATRACmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }

    set m $var(top).param.f.dec.m
    catch {destroy $m}

    menu $m -tearoff 0
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATDECCmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }
}

proc CATRACmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(ra) $val
    CATPlot $varname
}

proc CATDECCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(dec) $val
    CATPlot $varname
}

proc CATConfigRADEC {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    set var(ra) {}
    set var(dec) {}

    if {![CATValidDB $var(catdb)]} {
	return
    }

    if {[starbase_ncols $var(catdb)] < 2} {
	return
    }

    # first look for CDS named cols
    set ra "_RAJ2000"
    set dec "_DEJ2000"
    if {!([lsearch [starbase_columns $var(catdb)] $ra] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $dec] == -1)} {
	set var(ra) $ra
	set var(dec) $dec
    }

    if {$var(ra) != {} && $var(dec) != {}} {
	return
    }

    # next, look for an UCD (via VOTABLE)
    set db $var(catdb)
    upvar #0 $db T
    for {set cc 0} {$cc < $T(Ncols)} {incr cc} {
	if {[info exists ${db}(Ucd)]} {
	    switch -- [string tolower [string range [lindex $T(Ucd) $cc] 0 8]] {
		pos.eq.ra {set var(ra) [lindex $T(Header) $cc]}
		pos.eq.de {set var(dec) [lindex $T(Header) $cc]}
	    }
	}
    }

    if {$var(ra) != {} && $var(dec) != {}} {
	return
    }

    # ok, try to figure it out
    set radec {
 	"_RAB2000" "_DEB2000"
	"RAJ2000" "DEJ2000"
	"RA_J2000" "DEC_J2000"
	"RA (J2000)" "Dec (J2000)"
	"RA" "DEC"
 	"RA" "DECL"
 	"RA" "Dec"
	"RA(deg)" "DEC(deg)"
    }
    foreach {ra dec} $radec {
	if {!([lsearch [starbase_columns $var(catdb)] $ra] == -1) &&
	    !([lsearch [starbase_columns $var(catdb)] $dec] == -1)} {
	    set var(ra) $ra
	    set var(dec) $dec
	    return
	}

	# try lower case
	set ra [string tolower $ra]
	set dec [string tolower $dec]
	if {!([lsearch [starbase_columns $var(catdb)] $ra] == -1) &&
	    !([lsearch [starbase_columns $var(catdb)] $dec] == -1)} {
	    set var(ra) $ra
	    set var(dec) $dec
	    return
	}
    }

    # default
    set var(ra) [starbase_colname $var(catdb) 1]
    set var(dec) [starbase_colname $var(catdb) 2]

    return
}

# Other interface

proc CATOther {} {
    CATDialog catoth {} {} {} none
}

proc CATClearCurrentFrame {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) marker catalog delete all
    }
}


# Process Cmds

proc ProcessCatalogCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global cat

    # we need to be realized
    RealizeDS9

    set cvarname {}

    # determine catalog name
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	cds -
	cxc -
	ned -
	sdss -
	simbad {
	    # see if its from our list of cats
	    incr i
	    set which [string tolower [lindex $var $i]]
	    foreach mm $cat(def) {
		set ll [lindex $mm 0]
		set ww [lindex $mm 1]
		set ss [lindex $mm 2]
		set cc [lindex $mm 3]
		
		if {$ll != {-} && "cat${which}" == $ww} {
		    CATDialog $ww $ss $cc $ll sync
		    return
		}
	    }

	    # not a default, assume other name
	    switch -- $item {
		cds {CATDialog catoth $item $which $which sync}
		cxc {CATDialog catcsc cxc {Current Release} {Chandra Source} sync}
		ned {CATDialog catned ned ned {NED} sync}
		simbad {CATDialog catsimbad simbad simbad {SIMBAD} sync}
		sdss {CATDialog catsdss6 sdss 6 {SDSS Release 6} sync}
	    }

	    # clean up from above
	    switch -- $item {
		cds {}
		cxc -
		ned -
		simbad -
		sdss {incr i -1}
	    }

	    return
	}

	file -
	load {
	    incr i
	    set reader CATVOTRead
	    switch -- [lindex $var $i] {
		xml -
		vot {incr i; set reader CATVOTRead}
		sb -
		starbase {incr i; set reader starbase_read}
		csv -
		tsv {incr i; set reader CATTSVRead}
	    }

	    set fn [lindex $var $i]
	    if {$fn != {}} {
		CATDialog catoth {} {} {} none
		CATLoadFn [lindex $cat(cats) end] $fn $reader
		FileLast catfbox $fn
	    }
	    return
	}

	samp -
	name -
	coordinate -
	size -
	save -
	header -
	filter -
	clear -
	retrieve -
	cancel -
	print -
	close -
	server -
	symbol -
	sort -
	maxrows -
	allcols -
	allrows -
	ra -
	dec -
	show -
	hide -
	edit -
	panto {set cvarname [lindex $cat(cats) end]}

	default {
	    # is it an existing cat?
	    if {[lsearch $cat(cats) $item] == -1} {
		set which $item
		foreach mm $cat(def) {
		    set ll [lindex $mm 0]
		    set ww [lindex $mm 1]
		    set ss [lindex $mm 2]
		    set cc [lindex $mm 3]
		    
		    if {$ll != {-} && "cat${which}" == $ww} {
			CATDialog $ww $ss $cc $ll sync
			return
		    }
		}

		# not a default, assume other name
		switch -- $item {
		    cds {CATDialog catoth $item $which $which sync}
		}

		return
	    } else {
		set cvarname $item
		incr i
	    }
	}
    }

    # we should have a catalog now
    global $cvarname
    upvar #0 $cvarname cvar

    if {![info exists cvar(top)]} {
	Error "[msgcat::mc {Unable to find catalog window}] $cvarname"
	return
    }
    if {![winfo exists $cvar(top)]} {
	Error "[msgcat:: mc {Unable to find catalog window}] $cvarname"
	return
    }

    # now, process it
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	name {
	    incr i
	    set cvar(name) [lindex $var $i]
	}
	coordinate {
	    incr i
	    set cvar(x) [lindex $var $i]
	    incr i
	    set cvar(y) [lindex $var $i]
	    incr i
	    set cvar(sky) [lindex $var $i]
	}
	size {
	    incr i
	    set cvar(width) [lindex $var $i]
	    incr i
	    set cvar(height) [lindex $var $i]
	    incr i
	    set cvar(rformat) [lindex $var $i]
	}
	save {
	    incr i
	    set writer CATVOTWrite
	    switch -- [lindex $var $i] {
		xml -
		vot {incr i; set writer CATVOTWrite}
		sb -
		starbase {incr i; set writer starbase_write}
		csv -
		tsv {incr i; set writer CATTSVWrite}
	    }

	    set fn [lindex $var $i]
	    CATSaveFn $cvarname $fn $writer
	    FileLast catfbox $fn
	}
	header {}
	filter {
	    incr i
	    set item [lindex $var $i]
	    switch -- $item {
		load {
		    incr i
		    set fn [lindex $var $i]
		    if [catch {open $fn r} fp] {
			Error "[msgcat::mc {Unable to open file}] $fn: $fp"
			return
		    }
		    set flt [read -nonewline $fp]
		    catch {regsub {\n} $flt " " $flt}
		    set cvar(filter) [string trim $flt]
		    catch {close $fp}
		}
		default {
		    set cvar(filter) $item
		}
	    }
	    CATTable $cvarname
	}
	clear {CATOff $cvarname}
	retrieve -
	retreive {CATApply $cvarname 1}
	cancel {ARCancel $cvarname}
	print {CATPrint $cvarname}
	close {CATDestroy $cvarname}
	server {
	    incr i
	    set cat(server) [lindex $var $i]
	}
	symbol {
	    global $cvar(symdb)
	    set row 1
	    incr i
	    if {[string is integer [lindex $var $i]]} {
		set row [lindex $var $i]
		incr i
	    }

	    switch -- [lindex $var $i] {
		condition {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) condition] \
			[lindex $var $i]
		    CATPlot $cvarname
		}
		shape {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) shape] [lindex $var $i]
		    CATPlot $cvarname
		}
		color {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) color] [lindex $var $i]
		    CATPlot $cvarname
		}
		text {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) text] [lindex $var $i]
		    CATPlot $cvarname
		}
		size {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) size] [lindex $var $i]
		    CATPlot $cvarname
		}
		size2 {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) size2] [lindex $var $i]
		    CATPlot $cvarname
		}
		units {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) units] [lindex $var $i]
		    CATPlot $cvarname
		}
		angle {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) angle] [lindex $var $i]
		    CATPlot $cvarname
		}

		add {
		    set row [expr [starbase_nrows $cvar(symdb)]+1]
		    starbase_rowins $cvar(symdb) $row
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) shape] $cat(sym,shape)
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) color] $cat(sym,color)
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) units] $cat(sym,units)
		    CATPlot $cvarname
		}
		remove {
		    starbase_rowdel $cvar(symdb) $row
		    CATPlot $cvarname
		}
		save {
		    incr i
		    starbase_write $cvar(symdb) [lindex $var $i]
		}
		load {
		    incr i
		    set fn [lindex $var $i]
		    if {[file exists $fn]} {
			starbase_read $cvar(symdb) $fn
			CATPlot $cvarname
		    } else {
			Error "[msgcat::mc {Unable to open file}] $fn"
			return
		    }
		}
	    }
	}
	sort {
	    incr i
	    set cvar(sort) [lindex $var $i]
	    incr i
	    switch -- [lindex $var $i] {
		incr {
		    set cvar(sort,dir) "-increasing"
		}
		decr {
		    set cvar(sort,dir) "-decreasing"
		}
	    }
	    CATTable $cvarname
	}
	maxrows {
	    incr i
	    set cvar(max) [lindex $var $i]
	}
	allrows {
	    set cvar(allrows) 1
	}
	allcols {
	    set cvar(allcols) 1
	}
	show {
	    set cvar(show) 1
	    CATPlot $cvarname
	}
	hide {
	    set cvar(show) 0
	    CATPlot $cvarname
	}
	panto {
	    incr i
	    set cvar(panto) [FromYesNo [lindex $var $i]]
	}
	edit {
	    incr i
	    set cvar(edit) [FromYesNo [lindex $var $i]]
	    CATEdit $cvarname
	}
	ra {
	    incr i
	    set cvar(ra) [lindex $var $i]
	    CATPlot $cvarname
	}
	dec {
	    incr i
	    set cvar(dec) [lindex $var $i]
	    CATPlot $cvarname
	}
	samp {
	    global ds9
	    global samp

	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		send {
		    incr i
		    set name [string tolower [lindex $var $i]]
		    if {$ds9(samp)} {
			foreach arg $samp(apps,votable) {
			    foreach {key val} $arg {
				if {[string tolower $val] == $name} {
				    SAMPSendTableLoadVotable $key $cvarname
				    break
				}
			    }
			}
		    } else {
			Error [msgcat::mc {SAMP: not connected}]
		    }
		}
		broadcast {SAMPSendTableLoadVotable {} $cvarname}
		default {
		    SAMPSendTableLoadVotable {} $cvarname
		    incr i -1
		}
	    }
	}
    }
}

proc ProcessSendCatalogCmd {proc id param sock fn} {
    global cat

    set cc [lindex $cat(cats) end]
    switch -- [string tolower [lindex $param 0]] {
	header {}
	default {
	    set cc [lindex $param 0]
	    set param [lreplace $param 0 0]
	}
    }

    switch -- [string tolower [lindex $param 0]] {
	{} {$proc $id "$cat(cats)\n"}
	header {ProcessSend $proc $id $sock $fn {.txt} "[CATGetHeader $cc]\n"}
    }
}
