#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATCDSSrchDialog {varname} {
    upvar #0 $varname var
    global $varname

    global ds9
    global cat
    global catcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchDialog $varname"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # defaults
    # maybe modified if cat(vot)
    set catcdssrch(list,wave,param) {-kw.Wavelength}
    set catcdssrch(list,wave) [list none Radio IR optical UV EUV X-ray Gamma-ray]
    set catcdssrch(list,mission,param) {-kw.Mission}
    set catcdssrch(list,mission) [list none ANS ASCA BeppoSAX CGRO Chandra COBE Copernicus Einstein EUVE EXOSAT FAUST FUSE GINGA GRANAT HEAO Hipparcos HST HUT INTEGRAL IRAS ISO IUE MSX OAO-2 ORFEUS ROSAT RXTE SAS-1 SAS-2 SIRTF SMM SOHO Swift TD1 UIT ULYSSES WMAP WUPPE XMM]
    set catcdssrch(list,astro,param) {-kw.Astronomy}
    set catcdssrch(list,astro) [list none Abundances Ages AGN Associations Atomic_Data Binaries:cataclysmic Binaries:eclipsing Binaries:spectroscopic BL_Lac_objects Blue_objects Clusters_of_galaxies Constellations Diameters Earth Ephemerides Equivalent_widths Extinction Galaxies Galaxies:Markarian Galaxies:spectra Globular_Clusters Gravitational_lensing HII_regions Interstellar_Medium Magnetic_fields Masers Masses Models Multiple_Stars Nebulae Nonstellar Novae Obs_Log Open_Clusters Orbits Parallaxes Photometry Photometry:intermediate-band Photometry:narrow-band Photometry:surface Photometry:wide-band Planetary_Nebulae Planets+Asteroids Polarization Positional_Data Proper_Motions Pulsars QSOs Redshifts Rotational_Velocities Seyfert_Galaxies Spectral_Classification Spectrophotometry Spectroscopy Stars Stars:early-typeStars:Emission Stars:late-type Stars:peculiar Stars:variable Stars:white_dwarf Stars:WR Sun SuperNovae SuperNovae_Remnants Velocities YSOs]

    # AR variables
    set var(status) {}
    set var(sync) 0

    # CATCDSSrch variables
    set var(catdb) ${varname}catdb

    set var(source) {}
    set var(words) {}
    set var(wave) {}
    set var(mission) {}
    set var(astro) {}

    # top
    set w $var(top)
    set title [msgcat::mc {Search for Catalogs}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CATCDSSrchDestroy $varname"

    # menu
    $w configure -menu $var(mb)
    menu $var(mb)

    # apple
    AppleMenu ${varname}(mb)

    # file
    $var(mb) add cascade -label [msgcat::mc {File}] \
	-menu $var(mb).file
    menu $var(mb).file
    $var(mb).file add command -label [msgcat::mc {Retrieve}] \
	-command "CATCDSSrchApply $varname"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $var(mb).file add command -label [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    $var(mb).file add command -label [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "[msgcat::mc {Load}]..." \
	-command "CATCDSSrchLoadFile $varname"
    $var(mb).file add command -label "[msgcat::mc {Save}]..." \
	-command "CATCDSSrchSaveFile $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"

    # edit
    AREditMenu $varname

    # catalog server
    CATServerMenu $varname

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.tbl -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.name 
    label $w.param.or -text [msgcat::mc {Or}]
    frame $w.param.words
    frame $w.param.srch 
    pack $w.param.name $w.param.or $w.param.words -side top -anchor w -pady 4
    pack $w.param.srch -side bottom -fill both -expand true -anchor w -pady 4

    # param name
    label $w.param.name.title -text [msgcat::mc {Name or Designation}]
    entry $w.param.name.source -textvariable ${varname}(source) -width 45
    pack $w.param.name.title $w.param.name.source -side top -anchor w

    # param keywords
    label $w.param.words.title \
	-text [msgcat::mc {Words matching title, description}]
    entry $w.param.words.key -textvariable ${varname}(words) -width 45
    pack $w.param.words.title $w.param.words.key -side top -anchor w

    # param search
    frame $w.param.srch.wave
    frame $w.param.srch.mission
    frame $w.param.srch.astro
    pack $w.param.srch.wave $w.param.srch.mission $w.param.srch.astro \
	-side left -fill both -expand true -padx 4

    # param search wave
    frame $w.param.srch.wave.f
    label $w.param.srch.wave.title -text [msgcat::mc {Wavelength}]
    pack $w.param.srch.wave.title -side top -anchor w
    pack $w.param.srch.wave.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.wave.f.scroll \
	-command "$w.param.srch.wave.f.list yview"
    set ${varname}(listbox,wave) [listbox $w.param.srch.wave.f.list \
			      -yscroll "$w.param.srch.wave.f.scroll set" \
			      -setgrid 1 \
			      -width 15 \
			      -height 10 \
			      -selectmode browse \
			      -exportselection 0 \
			      -listvariable catcdssrch(list,wave)]
    grid $w.param.srch.wave.f.list $w.param.srch.wave.f.scroll -sticky news
    grid rowconfigure $w.param.srch.wave.f 0 -weight 1
    grid columnconfigure $w.param.srch.wave.f 0 -weight 1

    # param search mission
    frame $w.param.srch.mission.f
    label $w.param.srch.mission.title -text [msgcat::mc {Mission}]
    pack $w.param.srch.mission.title -side top -anchor w
    pack $w.param.srch.mission.f -side bottom -fill both -expand true  -anchor w

    scrollbar $w.param.srch.mission.f.scroll \
	-command "$w.param.srch.mission.f.list yview"
    set ${varname}(listbox,mission) [listbox $w.param.srch.mission.f.list \
				 -yscroll "$w.param.srch.mission.f.scroll set"\
				 -setgrid 1 \
				 -width 15 \
				 -height 10 \
				 -selectmode browse \
				 -exportselection 0 \
				 -listvariable catcdssrch(list,mission)]
    grid $w.param.srch.mission.f.list $w.param.srch.mission.f.scroll \
	-sticky news
    grid rowconfigure $w.param.srch.mission.f 0 -weight 1
    grid columnconfigure $w.param.srch.mission.f 0 -weight 1

    # param search astro
    frame $w.param.srch.astro.f
    label $w.param.srch.astro.title -text [msgcat::mc {Astronomy}]
    pack $w.param.srch.astro.title -side top -anchor w
    pack $w.param.srch.astro.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.astro.f.scroll \
	-command "$w.param.srch.astro.f.list yview"
    set ${varname}(listbox,astro) [listbox $w.param.srch.astro.f.list \
			       -yscroll "$w.param.srch.astro.f.scroll set" \
			       -setgrid 1 \
			       -width 15 \
			       -height 10 \
			       -selectmode browse \
			       -exportselection 0 \
			       -listvariable catcdssrch(list,astro)]
    grid $w.param.srch.astro.f.list $w.param.srch.astro.f.scroll -sticky news
    grid rowconfigure $w.param.srch.astro.f 0 -weight 1
    grid columnconfigure $w.param.srch.astro.f 0 -weight 1

    # tbl
    set var(tbl) [table $var(top).tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(catdb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $catcdssrch(mincols) \
		      -rows $catcdssrch(minrows) \
		      -colstretchmode all \
		      -width -1 \
		      -height -1 \
		      -maxwidth 400 \
		      -maxheight 190 \
		      -titlerows 1 \
		      -xscrollcommand [list $var(top).tbl.xscroll set]\
		      -yscrollcommand [list $var(top).tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -anchor w
		     ]
    #-browsecommand [list CATCDSSrchSelectCB $varname]

    scrollbar $var(top).tbl.yscroll -command [list $var(tbl) yview] \
	-orient vertical
    scrollbar $var(top).tbl.xscroll -command [list $var(tbl) xview] \
	-orient horizontal

    grid $var(tbl) $var(top).tbl.yscroll -sticky news
    grid $var(top).tbl.xscroll -stick news
    grid rowconfigure $var(top).tbl 0 -weight 1
    grid columnconfigure $var(top).tbl 0 -weight 1

    # status
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.item -anchor w -pady 4

    # buttons
    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "CATCDSSrchApply $varname"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.catalog -text [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.catalog \
	$w.buttons.clear $w.buttons.close -side left -padx 10 -expand true

    ARStatus $varname {}

    # initialize
    $var(listbox,wave) selection set 0
    $var(listbox,mission) selection set 0
    $var(listbox,astro) selection set 0
    $w.param.name.source select range 0 end

    # obtain dialog parameters
    if {$cat(vot)} {
	CATCDSSrchConfig $varname
    }
}

proc CATCDSSrchApply {varname} {
    upvar #0 $varname var
    global $varname

    global catcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchApply $varname"
    }

    set id [$var(listbox,wave) curselection]
    if {$id > 0} {
	set var(wave) [lindex $catcdssrch(list,wave) $id]
    } else {
	set var(wave) {}
    }
    set id [$var(listbox,mission) curselection]
    if {$id > 0} {
	set var(mission) [lindex $catcdssrch(list,mission) $id]
    } else {
	set var(mission) {}
    }
    set id [$var(listbox,astro) curselection]
    if {$id > 0} {
	set var(astro) [lindex $catcdssrch(list,astro) $id]
    } else {
	set var(astro) {}
    }

    ARApply $varname
    ARStatus $varname [msgcat::mc {Searching for catalogs}]

    CATCDSSrch $varname
}

proc CATCDSSrchDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchDestroy $varname"
    }

    if [info exists $var(catdb)] {
	unset $var(catdb)
    }

    ARDestroy $varname
}

proc CATCDSSrchConfig {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfig $varname"
    }

    set site [CATCDSURL]
    set cgidir {viz-bin}
    set script {votable}

    set var(url) "http://$site/$cgidir/$script"
    set var(query) [http::formatQuery -meta.aladin all]

    CATCDSSrchConfigLoad $varname
    return
}

proc CATCDSSrchConfigLoad {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfigLoad $varname"
    }

    set var(proc,done) CATCDSSrchConfigDone
    set var(proc,load) CATCDSSrchConfigLoad
    CATGetURL $varname
    return 
}

proc CATCDSSrchConfigDone {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATCDSSrchConfigDone $varname"
    }
}

proc CATCDSSrchConfigParse {t token} {
    upvar #0 $t T
    global $t
    global debug

    if {$debug(tcl,cat)} {
	set fp [open debug.xml w]
	puts $fp [http::data $token]
	close $fp
    }

    set xml [xml::parser \
		 -elementstartcommand [list CATCDSSrchConfigElemStartCB $t] \
		 -elementendcommand [list CATCDSSrchConfigElemEndCB $t] \
		 -ignorewhitespace 1 \
		]

    set T(tree,enable) 0
    set T(tree,state) {}
    if [catch {$xml parse [http::data $token]} err] {
	if {$debug(tcl,cat)} {
	    puts stderr "CATCDSSrchConfigParse: $err"
	}
    }
    unset ${t}(tree,enable)
    unset ${t}(tree,state)

    $xml free
}

proc CATCDSSrchConfigElemStartCB {t name attlist args} {
    upvar #0 $t T
    global $t
    global debug

    switch -- $name {
	RESOURCE {
	    set id {}
	    set type {}
	    foreach {key value} $attlist {
		switch -- [string tolower $key] {
		    type {set type "$value"}
		    id {set id "$value"}
		}
	    }
	    if {[string tolower $id] == "vizier"} {
		set T(tree,enable) 1

		global catcdssrch
		set catcdssrch(list,wave) [list none]
		set catcdssrch(list,mission) [list none]
		set catcdssrch(list,astro) [list none]
	    }
	}
	PARAM {
	    if {$T(tree,enable)} {
		set id {}
		set fname {}
		foreach {key value} $attlist {
		    switch -- [string tolower $key] {
			name {set fname "$value"}
			id {set id "$value"}
		    }
		}
		set T(tree,state) [string trim [string tolower $id]]
		switch -- $T(tree,state) {
		    wavelength {set catcdssrch(list,wave,param) $fname}
		    mission {set catcdssrch(list,mission,param) $fname}
		    astronomy {set catcdssrch(list,astro,param) $fname}
		}
		
	    }
	}
	VALUES {}
	OPTION {
	    if {$T(tree,enable)} {
		set item {}
		foreach {key value} $attlist {
		    switch -- [string tolower $key] {
			value {set item "$value"}
		    }
		}
		if {$item != {}} {
		    global catcdssrch
		    switch -- $T(tree,state) {
			wavelength {lappend catcdssrch(list,wave) $item}
			mission {lappend catcdssrch(list,mission) $item}
			astronomy {lappend catcdssrch(list,astro) $item}
		    }
		}
	    }
	}
    }

    return {}
}

proc CATCDSSrchConfigElemEndCB {t name args} {
    upvar #0 $t T
    global $t
    global debug

    # we can't count on this being called for all end-tags
    switch -- $name {
	RESOURCE {
	    # ok, we're done
	    set T(tree,enable) 0
	    return -code break
	}
    }

    return {}
}
