#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Table Commands

proc CATSelectCmd {varname ss rc} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATSelectCmd $varname $ss $rc"
    }

    if {$var(edit)} {
	CATSelectEditCmd $varname $ss $rc
    } else {
	CATSelectBrowseCmd $varname $ss $rc
    }
}

proc CATSelectEditCmd {varname ss rc} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATSelectEditCmd $varname $rc"
    }

    $var(frame) marker catalog $varname unselect

    set last [lindex [split $ss ,] 0]
    set next [lindex [split $rc ,] 0]
    
    if {[string is integer -strict $last]} {
	CATPlotUpdate $varname $last
    }
    
    if {[string is integer -strict $next]} {
	set mk "\{${varname}.${next}\}"

	# pan to first region
	if {$var(panto)} {
	    set tt [$var(frame) get marker catalog $mk tag]
	    if {$tt!={}} {
		set cc [$var(frame) get marker catalog $tt center wcs fk5]
		CATPanTo $varname [lindex $cc 0] [lindex $cc 1] wcs fk5
	    }
	}

	$var(frame) marker catalog $mk select
    }
}

proc CATSelectBrowseCmd {varname ss rc} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATSelectBrowseCmd $varname"
    }

    global $var(catdb)
    if {![CATValidDB $var(catdb)]} {
	return
    }

    $var(frame) marker catalog $varname unhighlite

    # init timer vars
    set var(blink,count) 0
    set var(blink,marker) {}

    # now see the current selection
    set last [lindex [split $ss ,] 0]
    set row [lindex [split $rc ,] 0]

    # for samp
    set rowlist {}

    foreach sel [$var(tbl) curselection] {
	set rr [lindex [split $sel ,] 0]

	set mk "\{${varname}.${rr}\}"
	if {[lsearch $var(blink,marker) $mk] == -1} {
	    lappend ${varname}(blink,marker) $mk
	    lappend rowlist [expr $rr-1]
	}
    }

    SAMPSendTableRowListCmd $varname $rowlist

    # pan to first region
    if {$var(panto)} {
	set ff [lindex $var(blink,marker) 0]
	if {$ff != {}} {
	    set tt [$var(frame) get marker catalog $ff tag]
	    if {$tt != {}} {
		set cc [$var(frame) get marker catalog $tt center wcs fk5]
		CATPanTo $varname [lindex $cc 0] [lindex $cc 1] wcs fk5
	    }
	}
    }

    # start timer, if needed
    if {!$var(blink)} {
	set var(blink) 1
	CATSelectTimer $varname
    }
}

proc CATSelectRows {varname rows} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATSelectRows $varname $rows"
    }

    if {![info exists ${varname}(top)]} {
	return
    }

    global $var(catdb)
    if {![CATValidDB $var(catdb)]} {
	return
    }

    if [info exists ${varname}(tbl)] {
	$var(tbl) selection clear all
	foreach rr $rows {
	    $var(tbl) selection set $rr,1
	}
	$var(tbl) see [lindex $rows 0],1
    }

    $var(frame) marker catalog $varname unhighlite

    # init timer vars
    set var(blink,count) 0
    set var(blink,marker) {}

    foreach sel [$var(tbl) curselection] {
	set rr [lindex [split $sel ,] 0]
	lappend ${varname}(blink,marker) "\{${varname}.${rr}\}"
    }

    # pan to first region
    if {$var(panto)} {
	set ff [lindex $var(blink,marker) 0]
	if {$ff != {}} {
	    set tt [$var(frame) get marker catalog $ff tag]
	    if {$tt != {}} {
		set cc [$var(frame) get marker catalog $tt center wcs fk5]
		CATPanTo $varname [lindex $cc 0] [lindex $cc 1] wcs fk5
	    }
	}
    }

    # start timer, if needed
    if {!$var(blink)} {
	set var(blink) 1
	CATSelectTimer $varname
    }
}

proc CATPanTo {varname x y sys sky} {
    upvar #0 $varname var
    global $varname

    PanToFrame $var(frame) $x $y $sys $sky
}

proc CATSelectTimer {varname} {
    upvar #0 $varname var
    global $varname

    switch -- $var(blink) {
	0 {
	    set var(blink) 0
	    set var(blink,count) 0
	    set var(blink,marker) {}
	}
	1 {
	    foreach mm $var(blink,marker) {
		$var(frame) marker catalog $mm highlite
	    }

	    incr ${varname}(blink,count)
	    if {$var(blink,count) < 5} {
		set var(blink) 2
	    } else {
		set var(blink) 0
	    }

	    after 250 [list CATSelectTimer $varname]
	}
	2 {
	    foreach mm $var(blink,marker) {
		$var(frame) marker catalog $mm unhighlite
	    }
	    set var(blink) 1

	    after 250 [list CATSelectTimer $varname]
	}
    }
}

# Marker Callbacks

proc CATHighliteCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATHighliteCB $tag $id"
    }

    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]

    upvar #0 $varname var
    global $varname

    if {![info exists ${varname}(top)]} {
	return
    }

    if {!$var(blink)} {
	if [info exists ${varname}(tbl)] {
	    $var(tbl) selection set $row,1
	    $var(tbl) see $row,1
	}
    }
}

proc CATUnhighliteCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATUnhighliteCB $tag $id"
    }

    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]

    upvar #0 $varname var
    global $varname

    if {![info exists ${varname}(top)]} {
	return
    }

    if {!$var(blink)} {
	if [info exists ${varname}(tbl)] {
	    $var(tbl) selection clear $row,1
	}
    }
}

proc CATEditCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATEditCB $tag $id"
    }

    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]
    set szcol [lindex $t 2]
    set sz2col [lindex $t 3]
    set units [lindex $t 4]
    set angcol [lindex $t 5]

    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if {![info exists ${varname}(top)]} {
	return
    }

    # shape
    set shape [$var(frame) get marker catalog $id type]

    if [catch {starbase_hdrget $var(tbldb) equinox} skyframe] {
	set skyframe fk5
	global errorInfo
	set errorInfo {}
    }
    switch -- $skyframe {
	1950.0 -
	B1950 {set skyframe fk4}
	2000.0 -
	J2000 -
	{} {set skyframe fk5}
    }

    # sys, skyformat
    switch -- $units {
	image -
	physical {
	    set sys $units
	    set skyformat degrees
	}
	degrees -
	arcmin -
	arcsec {
	    set sys wcs
	    set skyformat $units
	}
	default {
	    set sys physical
	    set skyformat degrees
	}
    }

    # get center and format
    switch $shape {
	circle {
	    if {$szcol>0} {
		starbase_set $var(tbldb) $row $szcol [$var(frame) get marker catalog $id $shape radius $sys $skyformat]
	    }
	}
	ellipse -
	box {
	    if {$szcol>0 && $sz2col>0} {
		set rr [$var(frame) get marker catalog $id $shape radius $sys $skyformat]
		starbase_set $var(tbldb) $row $szcol [lindex $rr 0]
		starbase_set $var(tbldb) $row $sz2col [lindex $rr 1]
	    }
	}
	vector {
	    if {$szcol>0} {
		set ll [$var(frame) get marker catalog $id $shape length $sys $skyformat]
		starbase_set $var(tbldb) $row $szcol $ll
	    }
	    
	    if {$angcol>0} {
		set ang [$var(frame) get marker catalog $id angle wcs $skyframe]
		starbase_set $var(tbldb) $row $angcol $ang
	    }
	}
	default {}
    }
}

proc CATMoveCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATMoveCB $tag $id"
    }

    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]

    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if {![info exists ${varname}(top)]} {
	return
    }

    # equinox
    if [catch {starbase_hdrget $var(tbldb) equinox} skyframe] {
	set skyframe fk5
	global errorInfo
	set errorInfo {}
    }
    switch -- $skyframe {
	1950.0 -
	B1950 {set skyframe fk4}
	2000.0 -
	J2000 -
	{} {set skyframe fk5}
    }

    # do we have formats for ra and dec?
    if [catch {starbase_hdrget $var(tbldb) UFMT} ff] {
	set ff {}
	global errorInfo
	set errorInfo {}
     }
    set raformat  [lindex $ff 0]
    set decformat [lindex $ff 1]

    # get center and format
    set coord [$var(frame) get marker catalog $id center wcs $skyframe degrees]
    set ra [lindex $coord 0]
    if {$raformat != {}} {
	set ra [uformat d $raformat $ra]
    }
    set dec [lindex $coord 1]
    if {$decformat != {}} {
	set dec [uformat d $decformat $dec]
    }

    starbase_set $var(tbldb) $row [starbase_colnum $var(tbldb) $var(ra)] $ra
    starbase_set $var(tbldb) $row [starbase_colnum $var(tbldb) $var(dec)] $dec
}    

proc CATRotateCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATRotateCB $tag $id"
    }

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATMoveCB $tag $id"
    }

    set t [split $tag .]
    set varname [lindex $t 0]
    set row [lindex $t 1]
    set angcol [lindex $t 2]

    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if {![info exists ${varname}(top)]} {
	return
    }

    # equinox
    if [catch {starbase_hdrget $var(tbldb) equinox} skyframe] {
	set skyframe fk5
	global errorInfo
	set errorInfo {}
    }
    switch -- $skyframe {
	1950.0 -
	B1950 {set skyframe fk4}
	2000.0 -
	J2000 -
	{} {set skyframe fk5}
    }

    # shape
    set shape [$var(frame) get marker catalog $id type]

    # get center and format
    switch $shape {
	ellipse -
	box {
	    if {$angcol>0} {
		set ang [$var(frame) get marker catalog $id angle wcs $skyframe]
		starbase_set $var(tbldb) $row $angcol $ang
	    }
	}
	default {}
    }
}

proc CATDeleteCB {tag id} {
    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATDeleteCB $tag $id"
    }
}

# Tcl Commands

proc CATButton {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # see if we are on a handle
    set h [$which get marker catalog handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker catalog $id edit begin $marker(handle)
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a segment of a polygon
    set h [$which get marker catalog polygon segment $x $y]
    set id [lindex $h 0]
    set segment [lindex $h 1]
    if {$segment} {
	$which marker catalog $id create polygon vertex $segment $x $y
	$which marker catalog $id edit begin $marker(handle)
	set marker(handle) [expr 4+$segment+1]
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a marker
    set id [$which get marker catalog id $x $y]
    if {$id != 0} {
	# select
	if {[$which get marker catalog $id property select]} {
	    $which marker catalog select only $x $y
	    $which marker catalog move begin $x $y
	    set marker(motion) beginMove
	    return
	}
	# highlite
	if {[$which get marker catalog $id property highlite]} {
	    $which marker catalog $id highlite only
	    $which marker catalog $id move back
	    set marker(motion) none
	    return
	}
    }

    # see if any markers are selected
    if {[$which get marker catalog select number]>0} {
	$which marker catalog unselect all
	set marker(motion) none
	return
    }

    # see if any markers are selected
    if {[$which get marker catalog highlite number]>0} {
	$which marker catalog unhighlite all
	set marker(motion) none
	return
    }

    set marker(handle) 0
    set marker(motion) none
}

proc CATShift {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # see if we are on a handle
    set h [$which get marker catalog handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker catalog $id rotate begin
	set marker(motion) beginRotate
	return
    }

    # else, see if we are on a marker
    if {[$which marker catalog select toggle $x $y]} {
	$which marker catalog move begin $x $y
	set marker(motion) beginMove
	return
    }

    if {[$which marker catalog highlite toggle $x $y]} {
	set marker(motion) none
	return
    }

    # else, start a region select
    $which region catalog select begin $x $y
    # $which region catalog highlite begin $x $y
    set marker(motion) shiftregion
}

proc CATMotion {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    switch -- $marker(motion) {
	none {}

	beginMove -
	move {
	    $which marker catalog move motion $x $y
	    set marker(motion) move
	}

	beginEdit -
	edit {
	    $which marker catalog edit motion $x $y $marker(handle)
	    set marker(motion) edit
	}

	beginRotate -
	rotate {
	    $which marker catalog rotate motion $x $y $marker(handle)
	    set marker(motion) rotate
	}

	region -
	shiftregion {
	    $which region catalog select motion $x $y
	    # $which region catalog highlite motion $x $y
	}
    }
}

proc CATRelease {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    switch -- $marker(motion) {
	none {}
	beginMove -
	beginRotate {}
	beginEdit {}
	move {$which marker catalog move end}
	edit {$which marker catalog edit end}
	rotate {$which marker catalog rotate end}
	region {
	    $which region catalog select end
	    $which region catalog catalog highlite end
	}
	shiftregion {
	    $which region catalog select shift end
	    $which region catalog highlite shift end
	}
    }

    unset marker(motion)
    unset marker(handle)

    # samp
    global ds9
    if {$ds9(samp)} {
	global samp
	if {$samp(apps,votable) != {}} {
	    set rr {}
	    foreach mm [$which get marker catalog highlite] {
		lappend rr [string trim \
				[lindex [$which get marker catalog $mm tag] 1]]
	    }
	    set rr [lsort $rr]
	    set varname {}
	    set rowlist {}
	    foreach ss $rr {
		set tt [split $ss {.}]
		set var [lindex $tt 0]
		set row [lindex $tt 1]
		if {$varname != $var} {
		    # dump what we have
		    SAMPSendTableRowListCmd $varname $rowlist

		    # now a new list
		    set varname $var
		    set rowlist [expr $row-1]
		} else {
		    lappend rowlist [expr $row-1]
		}
	    }
	    SAMPSendTableRowListCmd $varname $rowlist
	}
    }
}
