#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATDialog {varname format catalog title action} {
    global cat
   
    # first determine if aready in use, then increment
    if {[lsearch $cat(cats) $varname] >= 0} {
	incr cat(id)
	append varname $cat(id)
    }

    upvar #0 $varname var
    global $varname

    global ds9

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATDialog $varname :$format:$catalog:$title:$action"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    global current
    if {$current(frame) == {}} {
	return
    }

    # AR variables
    ARInit $varname CATServer

    # CAT variables
    lappend cat(cats) $varname

    set var(catdb) ${varname}catdb
    set var(tbldb) ${varname}tbldb
    set var(symdb) ${varname}symdb
    set var(symdl) ${varname}symdl
    set var(fltdl) ${varname}fltdl

    set var(frame) $current(frame)
    set var(system) $cat(system)
    set var(sky) $cat(sky)
    set var(skyformat) $cat(skyformat)
    set var(rformat) $cat(rformat)
    set var(width) $cat(width)
    set var(height) $cat(height)
    set var(max) $cat(max)
    set var(allrows) $cat(allrows)
    set var(allcols) $cat(allcols)
    set var(show) $cat(show)
    set var(edit) $cat(edit)
    set var(panto) $cat(panto)

    set var(blink) 0
    set var(blink,count) 0
    set var(blink,marker) {}

    CATSet $varname $format $catalog $title
    CATSymDBInit $varname

    # top
    set w $var(top)
    set wt [msgcat::mc {Catalog Tool}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $wt
    wm iconname $w $wt
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CATDestroy $varname"
    wm minsize $w 500 600

    # menu
    $w configure -menu $var(mb)
    menu $var(mb)

    # apple
    AppleMenu ${varname}(mb)

    # file
    $var(mb) add cascade -label [msgcat::mc {File}] -menu $var(mb).file
    menu $var(mb).file
    $var(mb).file add cascade -label [msgcat::mc {Load}] \
	-menu $var(mb).file.load
    $var(mb).file add cascade -label [msgcat::mc {Save}] \
	-menu $var(mb).file.save
    $var(mb).file add separator
    $var(mb).file add command \
	-label "[msgcat::mc {Display Catalog Header}]..." \
	-command "CATHeader $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Filter}] \
	-command "CATTable $varname"
    $var(mb).file add command -label [msgcat::mc {Clear}] \
	-command "CATOff $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Retrieve}] \
	-command "CATApply $varname 0"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $var(mb).file add separator
    $var(mb).file add cascade -label [msgcat::mc {SAMP}] \
	-menu $var(mb).file.samp
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Acknowledgment}] \
	-command "CATAck $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Update from Current Frame}] \
	-command "CATUpdate $varname"
    $var(mb).file add command \
	-label [msgcat::mc {Update from Current Crosshair}] \
	-command "CATCrosshair $varname"
    $var(mb).file add separator
	$var(mb).file add command -label [msgcat::mc {Copy to Regions}] \
	-command "CATPlotRegions $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
	-command "CATPageSetup $varname"
    $var(mb).file add command -label "[msgcat::mc {Print}]..." \
	-command "CATPrint $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "CATDestroy $varname"

    switch $ds9(wm) {
	x11 {
	    $var(mb).file entryconfig "[msgcat::mc {Page Setup}]..." \
		-state disabled
	}
	win32 {}
	aqua {}
    }

    menu $var(mb).file.load
    $var(mb).file.load add command -label "[msgcat::mc {VOTable}]..." \
	-command "CATLoadVOTFile $varname"
    $var(mb).file.load add command -label "[msgcat::mc {Starbase}]..." \
	-command "CATLoadSBFile $varname"
    $var(mb).file.load add command -label "[msgcat::mc {Tab-Separated-Value}]..." \
	-command "CATLoadTSVFile $varname"

    menu $var(mb).file.save
    $var(mb).file.save add command -label "[msgcat::mc {VOTable}]..." \
	-command "CATSaveVOTFile $varname"
    $var(mb).file.save add command -label "[msgcat::mc {Starbase}]..." \
	-command "CATSaveSBFile $varname"
    $var(mb).file.save add command -label "[msgcat::mc {Tab-Separated-Value}]..." \
	-command "CATSaveTSVFile $varname"

    SAMPMenu $var(mb).file.samp votable $varname

    # edit
    $var(mb) add cascade -label [msgcat::mc {Edit}] -menu $var(mb).edit
    menu $var(mb).edit
    $var(mb).edit add command -label [msgcat::mc {Cut}] \
	-command "CATCut $varname" -accelerator "${ds9(ctrl)}X"
    $var(mb).edit add command -label [msgcat::mc {Copy}] \
	-command "CATCopy $varname" -accelerator "${ds9(ctrl)}C"
    $var(mb).edit add command -label [msgcat::mc {Paste}] \
	-command "ARPaste $varname" -accelerator "${ds9(ctrl)}V"
    $var(mb).edit add command -label [msgcat::mc {Clear}] \
	-command "ARClear $varname"

    # catalog server
    CATServerMenu $varname

    # name server
    NSVRServerMenu $varname
    ARCoordMenu $varname
    ARSizeMenu $varname

    # symbol
    global $var(symdb)
    set flt $var(symdb)
    set sn [starbase_colnum $var(symdb) shape]
    set cn [starbase_colnum $var(symdb) color]

    $var(mb) add cascade -label [msgcat::mc {Symbol}] -menu $var(mb).symbol

    menu $var(mb).symbol
    $var(mb).symbol add cascade -label [msgcat::mc {Shape}] \
	-menu $var(mb).symbol.shape
    $var(mb).symbol add cascade -label [msgcat::mc {Color}] \
	-menu $var(mb).symbol.color
    $var(mb).symbol add separator
    $var(mb).symbol add command -label "[msgcat::mc {Advanced}]..." \
	-command "CATSymDialog $varname"

    menu $var(mb).symbol.shape
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Circle Point}] \
	-variable ${flt}(1,$sn) -value {circle point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Box Point}] \
	-variable ${flt}(1,$sn) -value {box point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Diamond Point}] \
	-variable ${flt}(1,$sn) -value {diamond point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Cross Point}] \
	-variable ${flt}(1,$sn) -value {cross point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {X Point}] \
	-variable ${flt}(1,$sn) -value {x point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Arrow Point}] \
	-variable ${flt}(1,$sn) -value {arrow point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {BoxCircle Point}]\
	-variable ${flt}(1,$sn) -value {boxcircle point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add separator
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Circle}] \
	-variable ${flt}(1,$sn) -value circle \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Ellipse}] \
	-variable ${flt}(1,$sn) -value ellipse \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Box}] \
	-variable ${flt}(1,$sn) -value box \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Text}] \
	-variable ${flt}(1,$sn) -value text \
	-command "CATPlot $varname"

    DefaultColorMenu $var(mb).symbol.color ${flt}(1,$cn) "CATPlot $varname"

    $var(mb) add cascade -label [msgcat::mc {Preferences}] -menu $var(mb).prefs
    menu $var(mb).prefs
    $var(mb).prefs add checkbutton -label [msgcat::mc {Show Regions}] \
	-variable ${varname}(show) -command "CATPlot $varname"
    $var(mb).prefs add checkbutton -label [msgcat::mc {Edit}] \
	-variable ${varname}(edit) -command "CATEdit $varname"
    $var(mb).prefs add checkbutton -label [msgcat::mc {Pan To}] \
	-variable ${varname}(panto)
    $var(mb).prefs add separator
    $var(mb).prefs add checkbutton -label [msgcat::mc {All Rows}] \
	-variable ${varname}(allrows)
    $var(mb).prefs add checkbutton -label [msgcat::mc {All Columns}] \
	-variable ${varname}(allcols)

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.tbl -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.f
    pack $w.param.f -side left

    label $w.param.f.ttitle -text [msgcat::mc {Catalog}]
    label $w.param.f.title -textvariable ${varname}(title) \
	-relief groove -width 60 -anchor w

    label $w.param.f.tcat -text [msgcat::mc {Identification}]
    label $w.param.f.cat -textvariable ${varname}(catalog) \
	-relief groove -width 60 -anchor w

    label $w.param.f.nametitle -text [msgcat::mc {Object}]
    entry $w.param.f.name -textvariable ${varname}(name) -width 60

    label $w.param.f.sky -textvariable ${varname}(sky) -anchor w
    set var(xname) [label $w.param.f.xtitle -text {} \
			-font {symbol 12} -width 1]
    entry $w.param.f.x -textvariable ${varname}(x) -width 14
    set var(yname) [label $w.param.f.ytitle -text {} \
			-font {symbol 12} -width 1]
    entry $w.param.f.y -textvariable ${varname}(y) -width 14
    label $w.param.f.skyformat -textvariable ${varname}(skyformat)

    label $w.param.f.wtitle -text [msgcat::mc {Width}]
    entry $w.param.f.w -textvariable ${varname}(width) -width 14
    label $w.param.f.htitle -text [msgcat::mc {Height}]
    entry $w.param.f.h -textvariable ${varname}(height) -width 14
    label $w.param.f.format -textvariable ${varname}(rformat)

    label $w.param.f.mfilter -text [msgcat::mc {Filter}]
    entry $w.param.f.filter -textvariable ${varname}(filter) -width 40
    button $w.param.f.bfilter -text [msgcat::mc {Edit}] \
	-command "CATEditDialog $varname filter $var(catdb)"

    label $w.param.f.msort -text [msgcat::mc {Sort}]
    menubutton $w.param.f.sort -textvariable ${varname}(sort) \
	-menu $w.param.f.sort.m -relief raised -width 14
    radiobutton $w.param.f.isort -text [msgcat::mc {Increase}] \
	-variable ${varname}(sort,dir) -value "-increasing" \
	-command "CATTable $varname"
    radiobutton $w.param.f.dsort -text [msgcat::mc {Decrease}] \
	-variable ${varname}(sort,dir) -value "-decreasing" \
	-command "CATTable $varname"

    label $w.param.f.mtitle -text [msgcat::mc {Max Rows}]
    entry $w.param.f.max -textvariable ${varname}(max) -width 14

    checkbutton $w.param.f.allrows -text [msgcat::mc {All Rows}] \
	-variable ${varname}(allrows)

    checkbutton $w.param.f.allcols -text [msgcat::mc {All Columns}] \
	-variable ${varname}(allcols)

    label $w.param.f.ftitle -text [msgcat::mc {Found}] 
    set var(found) [label $w.param.f.found -relief groove -width 14 -anchor w]

    label $w.param.f.mra -text [msgcat::mc {RA}]
    menubutton $w.param.f.ra -textvariable ${varname}(ra) \
	-menu $w.param.f.ra.m -relief raised -width 14
    label $w.param.f.mdec -text [msgcat::mc {DEC}]
    menubutton $w.param.f.dec -textvariable ${varname}(dec) \
	-menu $w.param.f.dec.m -relief raised -width 14

    grid rowconfigure $w.param.f 0 -pad 4
    grid rowconfigure $w.param.f 1 -pad 4
    grid rowconfigure $w.param.f 2 -pad 4
    grid rowconfigure $w.param.f 3 -pad 4
    grid rowconfigure $w.param.f 4 -pad 4
    grid rowconfigure $w.param.f 5 -pad 4
    grid rowconfigure $w.param.f 6 -pad 4
    grid rowconfigure $w.param.f 7 -pad 4

    grid $w.param.f.ttitle x $w.param.f.title - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.tcat x $w.param.f.cat - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.nametitle x $w.param.f.name - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.sky $w.param.f.xtitle $w.param.f.x $w.param.f.ytitle \
	$w.param.f.y $w.param.f.skyformat -padx 4 -pady 1 -sticky w
    grid $w.param.f.wtitle x $w.param.f.w $w.param.f.htitle \
	$w.param.f.h $w.param.f.format -padx 4 -pady 1 -sticky w
    grid $w.param.f.mfilter x $w.param.f.filter - - $w.param.f.bfilter \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.msort x $w.param.f.sort $w.param.f.isort $w.param.f.dsort \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.mtitle x $w.param.f.max $w.param.f.ftitle $w.param.f.found \
	$w.param.f.allrows -padx 4 -pady 1 -sticky w
    grid $w.param.f.mra x $w.param.f.ra $w.param.f.mdec $w.param.f.dec \
	$w.param.f.allcols -padx 4 -pady 1 -sticky w

    # table
    set var(tbl) [table $var(top).tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(tbldb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $cat(mincols) \
		      -rows $cat(minrows) \
		      -width -1 \
		      -height -1 \
		      -maxwidth 300 \
		      -maxheight 300 \
		      -titlerows 1 \
		      -xscrollcommand [list $var(top).tbl.xscroll set]\
		      -yscrollcommand [list $var(top).tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -anchor w \
		      -browsecommand [list CATSelectCmd $varname %s %S] \
		     ]

    scrollbar $var(top).tbl.yscroll -command [list $var(tbl) yview] \
	-orient vertical
    scrollbar $var(top).tbl.xscroll -command [list $var(tbl) xview] \
	-orient horizontal

    grid $var(tbl) $var(top).tbl.yscroll -sticky news
    grid $var(top).tbl.xscroll -stick news
    grid rowconfigure $var(top).tbl 0 -weight 1
    grid columnconfigure $var(top).tbl 0 -weight 1

    # status
    label $w.status.title -text [msgcat::mc {Status}]
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.title $w.status.item -side left -anchor w -pady 4

    # buttons
    button $w.buttons.filter -text [msgcat::mc {Filter}] \
	-command "CATTable $varname"
    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "CATApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command "CATOff $varname"
    set var(samp) [button $w.buttons.samp -text [msgcat::mc {SAMP}] \
		       -command "SAMPSendTableLoadVotable {} $varname"]
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "CATDestroy $varname"
    pack $w.buttons.filter $w.buttons.apply $w.buttons.cancel \
	$w.buttons.clear $w.buttons.samp $w.buttons.close \
	-side left -padx 10 -expand true

    # needs to go after sort menu button is defined
    CATSortMenu $varname
    CATRADECMenu $varname
    switch $var(format) {
	cds {$var(mb) entryconfig [msgcat::mc {Catalog Server}] -state normal}
	cxc -
	ned -
	sdss -
	simbad {$var(mb) entryconfig [msgcat::mc {Catalog Server}] -state disabled}
    }

    ARCoord $varname
    CATUpdate $varname
    CATDialogUpdate $varname

    ARStatus $varname {}

    switch -- $action {
	apply {CATApply $varname 0}
	sync {CATApply $varname 1}
	db {CATDB $varname}
	none {}
    }
}

proc CATDialogUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATDialogUpdate $varname"
    }

    SAMPUpdateMenu $var(mb).file.samp votable $varname
    SAMPUpdateButton $var(mb).file.samp votable $varname
}

proc CATAck {varname} {
    upvar #0 $varname var
    global $varname

    switch $var(format) {
	cds {CATCDSAck $varname}
	cxc {CATCXCAck $varname}
	ned {CATNEDAck $varname}
	sdss {CATSDSSAck $varname}
	simbad {CATSIMBADAck $varname}
    }
}

proc CATApply {varname sync} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATApply $varname $sync"
    }

    if {$var(catalog) == {}} {
	Error [msgcat::mc {No Catalog specified}]
	return
    }

    set var(sync) $sync
    ARApply $varname
    if {$var(name) != {}} {
	set var(system) wcs
	set var(sky) fk5
	ARCoord $varname

	NSVRServer $varname
    } else {
	CATServer $varname
    }
}

proc CATCopy {varname} {
    upvar #0 $varname var
    global $varname

    set w [focus -displayof $var(top)]
    if {$w == $var(tbl)} {
	CATCopyTable $varname
    } else {
	tk_entryCopy $w
    }
}

proc CATCopyTable {varname} {
    upvar #0 $varname var
    global $varname

    set w [focus -displayof $var(top)]

    set sel [$var(tbl) curselection]
    set data {}
    set row [lindex [split [lindex $sel 0] ,] 0]
    foreach ss $sel {
	set rr [lindex [split $ss ,] 0]
	if {$rr != $row} {
	    append data "\n"
	    set row $rr
	} else {
	    if {$data != {}} {
		append data "\t"
	    }
	}
	append data "[$var(tbl) get $ss]"
    }
    append data "\n"
    clipboard clear -displayof $w
    clipboard append -displayof $w $data
}

proc CATCut {varname} {
    upvar #0 $varname var
    global $varname

    set w [focus -displayof $var(top)]
    if {$w == $var(tbl)} {
	CATCopyTable $varname
    } else {
	tk_entryCut $w
    }
}

proc CATCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    if {[$var(frame) has wcs equatorial $var(system)]} {
	set coord [$var(frame) get crosshair \
		       $var(system) $var(sky) sexagesimal]
	set var(x) [lindex $coord 0]
	set var(y) [lindex $coord 1]
	set var(name) {}
    }
}

proc CATDB {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(width) {}
    set var(height) {}

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

proc CATDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global $var(tbldb)
    global $var(symdb)
    global $var(symdl)
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATDestroy $varname"
    }

    upvar #0 $var(symdl) svar
    if [info exists svar(top)] {
	if [winfo exists $svar(top)] {
	    CATSymDestroy $var(symdl)
	}
    }

    if [info exists $var(symdb)] {
	unset $var(symdb)
    }
    if [info exists $var(tbldb)] {
	unset $var(tbldb)
    }
    if [info exists $var(catdb)] {
	unset $var(catdb)
    }

    set i [lsearch $cat(cats) $varname]
    if {$i>=0} {
	set cat(cats) [lreplace $cat(cats) $i $i]
    }

    ARDestroy $varname
}

proc CATEdit {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global current

    $var(frame) marker catalog unselect all
    $var(frame) marker catalog unhighlite all

    CATPlot $varname

    if {$var(edit)} {
	$var(tbl) configure \
	    -state normal \
	    -selectmode single
    } else {
	$var(tbl) configure \
	    -state disabled \
	    -selectmode extended
    }
}

proc CATGetHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    set t $var(tbldb)
    upvar #0 $t T

    if [CATValidDB $var(tbldb)] {
	set hdr {}

	# header
	set nl [expr $T(HLines)-2]
	for {set ll 1} {$ll <= $nl} {incr ll} {
	    append hdr "$T(H_$ll)\n"
	}
	append hdr "\n"

	# dump cols stats
	set nc $T(Ncols)
	for {set cc 1} {$cc <= $nc} {incr cc} {
	    append hdr "# name=[lindex $T(Header) [expr $cc-1]] "
	    if {[info exists ${t}(DataType)]} {
		append hdr "datatype=[lindex $T(DataType) [expr $cc-1]] "
	    }

	    if {[info exists ${t}(Id)]} {
		if {[lindex $T(Id) [expr $cc-1]] != {}} {
		    append hdr "id=[lindex $T(Id) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(ArraySize)]} {
		if {[lindex $T(ArraySize) [expr $cc-1]] != {}} {
		    append hdr "arraysize=[lindex $T(ArraySize) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Width)]} {
		if {[lindex $T(Width) [expr $cc-1]] != {}} {
		    append hdr "width=[lindex $T(Width) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Precision)]} {
		if {[lindex $T(Precision) [expr $cc-1]] != {}} {
		    append hdr "precision=[lindex $T(Precision) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Unit)]} {
		if {[lindex $T(Unit) [expr $cc-1]] != {}} {
		    append hdr "unit=[lindex $T(Unit) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Ref)]} {
		if {[lindex $T(Ref) [expr $cc-1]] != {}} {
		    append hdr "ref=[lindex $T(Ref) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Ucd)]} {
		if {[lindex $T(Ucd) [expr $cc-1]] != {}} {
		    append hdr "ucd=[lindex $T(Ucd) [expr $cc-1]] "
		}
	    }

	    if {[info exists ${t}(Description)]} {
		if {[lindex $T(Description) [expr $cc-1]] != {}} {
		    append hdr "[lindex $T(Description) [expr $cc-1]] "
		}
	    }
	    append hdr "\n"
	}

	return $hdr
    }
    return {}
}

proc CATHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    SimpleTextDialog ${varname}hdr "$var(title) [msgcat::mc {Header}]" \
	80 20 insert top [CATGetHeader $varname]
}

proc CATKey {which key} {
    global ds9
    global cat

    set cat(key) $key
    set cat(key,update) {}

    $which marker catalog key
    foreach rr $cat(key,update) {
	eval "CATPlotUpdate [lindex $rr 0] [lindex $rr 1]"
    }

    unset cat(key)
    unset cat(key,update)
}

proc CATPageSetup {varname} {
    upvar #0 $varname var
    global $varname

    global ds9
    switch $ds9(wm) {
	x11 {}
	win32 {win32 pm pagesetup}
	aqua {macosx pm pagesetup}
    }
}

proc CATPrint {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global ds9
    switch $ds9(wm) {
	x11 {CATPSPrint $varname}
	win32 {win32 pm print text [::textutil::tabify::untabify2 [starbase_write_ $var(tbldb)] 12]}
	aqua {macosx pm print text [::textutil::tabify::untabify2 [starbase_write_ $var(tbldb)] 12]}
    }	
}

proc CATPSPrint {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global ps

    if {[PSPrintDialog txt]} { 
	if {$ps(dest) == "file"} {
	    catch {set ch [open "| cat > $ps(filename)" w]}
	} else {
	    catch {set ch [open "| $ps(cmd)" w]}
	}

	if {$ch != {}} {
	    starbase_writefp $var(tbldb) $ch
	    close $ch
	} else {
	    Error [msgcat::mc {An error has occurred while printing}]
	    return
	}
    }
}

proc CATServer {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATServer $varname"
    }

    if {($var(x) != {}) && 
	($var(y) != {}) && 
	($var(width) != {}) && 
	($var(height) != {})} {

	ARStatus $varname "Searching [string range $var(title) 0 50]"

	switch $var(format) {
	    cds {CATCDS $varname}
	    cxc {CATCXC $varname}
	    ned {CATNED $varname}
	    sdss {CATSDSS $varname}
	    simbad {CATSIMBAD $varname}
	}
    } else {
	ARStatus $varname [msgcat::mc {Please specify width, height, and either name or (ra,dec)}]

	$var(mb).file entryconfig [msgcat::mc {Retrieve}] -state normal
	$var(mb).file entryconfig [msgcat::mc {Cancel}] -state disabled

	$var(apply) configure -state normal
	$var(cancel) configure -state disabled
    }
}

proc CATUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts stderr "CATUpdate $varname"
    }

    if {[$var(frame) has wcs equatorial $var(system)]} {
	set coord [$var(frame) get fits center \
		       $var(system) $var(sky) $var(skyformat)]
	set var(x) [lindex $coord 0]
	set var(y) [lindex $coord 1]

	set size [$var(frame) get fits size \
		      $var(system) $var(sky) $var(rformat)]
	set var(width) [lindex $size 0]
	set var(height) [lindex $size 1]
	set var(name) {}
    }
}

# Edit Dialog

proc CATEditDialog {varname which db} {
    upvar #0 $varname var
    global $varname
    global ds9
    global ed

    set w ".${varname}edit"
    set mb ".${varname}editmb"

    set ed(ok) 0
    set ed(text) $w.param.txt

    DialogCreate $w [msgcat::mc {Edit}]

    $w configure -menu $mb
    menu $mb

    # apple
    AppleMenu mb

    # file
    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command {set ed(ok) 1}
    $mb.file add separator
    $mb.file add command -label "[msgcat::mc {Load}]..." -command "CATEditLoad"
    $mb.file add command -label "[msgcat::mc {Save}]..." -command "CATEditSave"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Cancel}] -command {set ed(ok) 0}

    # edit
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Undo}] \
	-command "$ed(text) edit undo" -accelerator "${ds9(ctrl)}Z"
    $mb.edit add command -label [msgcat::mc {Redo}] \
	-command "$ed(text) edit redo" -accelerator "${ds9(shiftctrl)}Z"
    $mb.edit add separator
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command "tk_textCut $ed(text)" -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command "tk_textCopy $ed(text)" -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command "tk_textPaste $ed(text)" -accelerator "${ds9(ctrl)}V"

    global $db
    # column
    $mb add cascade -label [msgcat::mc {Column}] -menu $mb.col
    if [info exists $mb.col] {
	destroy $mb.col)
    }
    menu $mb.col
    if [CATValidDB $db] {
	set cnt -1
	foreach col [starbase_columns $db] {
	    $mb.col add command -label "$col" \
		-command "$ed(text) insert insert \{\$$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$mb.col entryconfig $col -columnbreak 1
	    }
	}
    }

    # operator
    $mb add cascade -label [msgcat::mc {Operator}] -menu $mb.op
    menu $mb.op
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add command -label {!} \
	-command "$ed(text) insert insert {!}"
    $mb.op add command -label {(} \
				   -command "$ed(text) insert insert {(}"
	$mb.op add command -label {)} \
	-command "$ed(text) insert insert {)}"
    $mb.op add separator
    $mb.op add command -label {*} \
	-command "$ed(text) insert insert {*}"
    $mb.op add command -label {/} \
	-command "$ed(text) insert insert {/}"
    $mb.op add command -label {%} \
	-command "$ed(text) insert insert {%}"
    $mb.op add command -label {+} \
	-command "$ed(text) insert insert {+}"
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add separator
    $mb.op add command -label {<} \
	-command "$ed(text) insert insert {<}"
    $mb.op add command -label {>} \
	-command "$ed(text) insert insert {>}"
    $mb.op add command -label {<=} \
	-command "$ed(text) insert insert {<=}"
    $mb.op add command -label {>=} \
	-command "$ed(text) insert insert {>=}"
    $mb.op add command -label {==} \
	-command "$ed(text) insert insert {==}"
    $mb.op add command -label {!=} \
	-command "$ed(text) insert insert {!=}"
    $mb.op add separator
    $mb.op add command -label {&&} \
	-command "$ed(text) insert insert {&&}"
    $mb.op add command -label {||} \
	-command "$ed(text) insert insert {||}"

    # operator
    $mb add cascade -label [msgcat::mc {Math Function}] -menu $mb.math
    menu $mb.math
    $mb.math add command -label {acos} \
	-command "$ed(text) insert insert {acos()}"
    $mb.math add command -label {asin} \
	-command "$ed(text) insert insert {asin()}"
    $mb.math add command -label {atan} \
	-command "$ed(text) insert insert {atan()}"
    $mb.math add command -label {atan2} \
	-command "$ed(text) insert insert {atan2(,)}"
    $mb.math add command -label {ceil} \
	-command "$ed(text) insert insert {ceil()}"
    $mb.math add command -label {cos} \
	-command "$ed(text) insert insert {cos()}"
    $mb.math add command -label {cosh} \
	-command "$ed(text) insert insert {cosh()}"
    $mb.math add command -label {exp} \
	-command "$ed(text) insert insert {exp()}"
    $mb.math add command -label {floor} \
	-command "$ed(text) insert insert {floor()}"
    $mb.math add command -label {fmod} \
	-command "$ed(text) insert insert {fmod(,)}"
    $mb.math add command -label {hypot} \
	-command "$ed(text) insert insert {hypot(,)}"
    $mb.math add command -label {log} \
	-command "$ed(text) insert insert {log()}"
    $mb.math add command -label {log10} \
	-command "$ed(text) insert insert {log10()}"
    $mb.math add command -label {pow} \
	-command "$ed(text) insert insert {pow(,)}"
    $mb.math add command -label {sin} \
	-command "$ed(text) insert insert {sin()}"
    $mb.math add command -label {sinh} \
	-command "$ed(text) insert insert {sinh()}"
    $mb.math add command -label {sqrt} \
	-command "$ed(text) insert insert {sqrt()}"
    $mb.math add command -label {tan} \
	-command "$ed(text) insert insert {tan()}"
    $mb.math add command -label {tanh} \
	-command "$ed(text) insert insert {tanh()}"
    $mb.math add command -label {abs} \
	-command "$ed(text) insert insert {abs()}"
    $mb.math add command -label {double} \
	-command "$ed(text) insert insert {double()}"
    $mb.math add command -label {int} \
	-command "$ed(text) insert insert {int()}"
    $mb.math add command -label {round} \
	-command "$ed(text) insert insert {round()}"

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    text $w.param.txt \
	-height 10 \
	-width 60 \
	-font {courier 12} \
	-yscrollcommand "$w.param.yscroll set" \
	-xscrollcommand "$w.param.xscroll set" \
	-undo true \
	-wrap none 

    scrollbar $w.param.yscroll -command [list $ed(text) yview] \
	-orient vertical
    scrollbar $w.param.xscroll -command [list $ed(text) xview] \
	-orient horizontal

    grid $ed(text) $w.param.yscroll -sticky news
    grid $w.param.xscroll -stick news
    grid rowconfigure $w.param 0 -weight 1
    grid columnconfigure $w.param 0 -weight 1

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok $w.buttons.cancel -side left -padx 10 -expand true

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    $ed(text) insert end $var($which)
    $ed(text) see end

    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok

    if {$ed(ok)} {
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	set var($which) [string trim $flt]
    }

    DialogDismiss $w
    destroy $mb

    set rr $ed(ok)
    unset ed
    return $rr
}

proc CATEditSave {} {
    global ed

    set fn [SaveFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn w} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	puts $fp [string trim $flt]
	catch {close $fp}
    }
}

proc CATEditLoad {} {
    global ed

    set fn [OpenFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn r} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	$ed(text) delete 1.0 end
	$ed(text) insert end [read -nonewline $fp]
	$ed(text) see end
	catch {close $fp}
    }
}

proc UpdateCATDialog {} {
    global cat

    foreach varname $cat(cats) {
	CATDialogUpdate $varname
    }
}