#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CentroidDef {} {
    global centroid

    set centroid(top) .centroid
    set centroid(mb) .centroidmb

    set centroid(iteration) 5
    set centroid(radius) 10
}

proc CentroidDialog {} {
    global centroid
    global ds9

    # see if we already have a window visible

    if [winfo exists $centroid(top)] {
	raise $centroid(top)
	return
    }

    set w $centroid(top)
    set title [msgcat::mc {Centroid Parameters}]
    set length 300

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW CentroidDestroyDialog
    wm minsize $w 375 100

    $w configure -menu $centroid(mb)

    menu $centroid(mb)

    AppleMenu centroid(mb)

    $centroid(mb) add cascade -label [msgcat::mc {File}] \
	-menu $centroid(mb).file

    menu $centroid(mb).file
    $centroid(mb).file add command -label [msgcat::mc {Apply}] \
	-command CentroidApplyDialog
    $centroid(mb).file add separator
    $centroid(mb).file add command -label [msgcat::mc {Close}] \
	-command CentroidDestroyDialog

    UpdateCentroidDialog

    frame $w.cb -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.cb $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.cb.iteration
    scale $w.cb.iteration.slider -from 1 -to 11 -length $length \
	-variable centroid(iteration) -orient horizontal \
	-label [msgcat::mc {Iteration}] \
	-tickinterval 2 -showvalue false -resolution 1 \
	-command CentroidApplyDialog
    entry $w.cb.iteration.value -textvariable centroid(iteration) -width 7
    pack $w.cb.iteration.slider $w.cb.iteration.value -side left -padx 4
    pack $w.cb.iteration.value

    frame $w.cb.radius
    scale $w.cb.radius.slider -from 0 -to 50 -length $length \
	-variable centroid(radius) -orient horizontal \
	-label [msgcat::mc {Radius}] \
	-tickinterval 10 -showvalue false -resolution .5 \
	-command CentroidApplyDialog
    entry $w.cb.radius.value -textvariable centroid(radius) -width 7
    pack $w.cb.radius.slider $w.cb.radius.value -side left -padx 4
    pack $w.cb.radius.value

    pack $w.cb.iteration $w.cb.radius -side top

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command CentroidApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command CentroidDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true
}

proc CentroidDestroyDialog {} {
    global centroid

    if {[winfo exists $centroid(top)]} {
	destroy $centroid(top)
	destroy $centroid(mb)
    }
}

proc CentroidApplyDialog {{value {}}} {
    # value is ignored
    global centroid
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker centroid option \
	    $centroid(iteration) $centroid(radius)
    }
}

proc UpdateCentroidDialog {} {
    global centroid
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateCentroidDialog"
    }

    if {[winfo exists $centroid(top)]} {
	if {$current(frame) != {}} {
	    set cc [$current(frame) get marker centroid option]
	    set centroid(iteration) [lindex $cc 0]
	    set centroid(radius) [lindex $cc 1]
	}
    }
}
