#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ColorbarDef {} {
    global ds9
    global colorbar
    global pcolorbar

    set colorbar(top) .clrbar
    set colorbar(mb) .clrbarmb

    set colorbar(thick) 20

    # these will be recalculated
    set colorbar(horizontal,gap) .9
    set colorbar(vertical,gap) .6
    set colorbar(vertical,width) 75
    set colorbar(horizontal,height) 45
    set colorbar(width) 512
    set colorbar(height) 45

    set colorbar(num) 64
    set colorbar(start) $ds9(menu,start)
    set colorbar(count) $ds9(menu,start)
    set colorbar(cmaps) 0
    set colorbar(lock) 0
    set colorbar(private) false
    set colorbar(frame) {}

    set colorbar(map) Grey
    set colorbar(invert) 0
    set colorbar(numerics) 1
    set colorbar(orientation) horizontal
    set colorbar(font) helvetica
    set colorbar(font,size) 10
    set colorbar(font,style) normal

    set pcolorbar(map) $colorbar(map)
    set pcolorbar(invert) $colorbar(invert)
    set pcolorbar(numerics) $colorbar(numerics)
    set pcolorbar(orientation) $colorbar(orientation)
    set pcolorbar(font) $colorbar(font)
    set pcolorbar(font,size) $colorbar(font,size)
    set pcolorbar(font,style) $colorbar(font,style)
}

proc CreateColorbar {} {
    global ds9
    global colorbar
    global canvas
    global view

    switch -- $ds9(visual) {
	pseudocolor {
	    $ds9(canvas) create colorbar$ds9(visual)$ds9(depth) \
		-private $colorbar(private) \
		-width $colorbar(width) \
		-height $colorbar(height) \
		-tag colorbar \
		-orientation 0 \
		-size $colorbar(thick) \
		-numopts [ColorbarOpts]

	    # And set the two windows that where created before the 
	    # colorbar was created, to the new colormap

	    colorbar set colormap window "$ds9(main)"

	    # This is a kluge, but it works. We want to reassign the bg color 
	    # for $ds9(main) from the old colormap to the new colormap
	    
	    $ds9(main) configure -background [$ds9(main) cget -background]
	}
	truecolor {
	    $ds9(canvas) create colorbar$ds9(visual)$ds9(depth) \
		-width $colorbar(width) \
		-height $colorbar(height) \
		-colors 2048 -tag colorbar \
		-orientation 0 \
		-size $colorbar(thick) \
		-numopts [ColorbarOpts]
	    $ds9(canvas) create colorbarrgb$ds9(visual)$ds9(depth) \
		-width $colorbar(width) \
		-height $colorbar(height) \
		-colors 2048 -tag colorbarrgb \
		-orientation 0 \
		-size $colorbar(thick) \
		-numopts [ColorbarOpts]
	}
    }
}

proc InitColorbar {} {
    global current
    global colorbar

    set current(colorbar) colorbar

    $current(colorbar) map "{$colorbar(map)}"
    $current(colorbar) invert $colorbar(invert)
}

proc ResetColormap {} {
    global current
    global colorbar
    global rgb

    $current(colorbar) reset
    if {$current(frame) != {} } {
	RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap [$current(colorbar) get colormap]"
	set colorbar(invert) [$current(colorbar) get invert]
    }

    UpdateColorDialog
}

proc LoadColormap {} {
    LoadColormapFile [OpenFileDialog colorbarfbox]
}

proc LoadColormapFile {filename} {
    global current
    global colorbar
    global ds9

    if {$filename != {}} {
	colorbar load "\{$filename\}"
	set id [colorbar get id]
	set colorbar(map) [colorbar get name]

	$ds9(mb).color insert $colorbar(count) radiobutton \
	    -label "$colorbar(map)" \
	    -variable colorbar(map) -command "ChangeColormapID $id"

	if {[winfo exists $colorbar(top)]} {
	    $colorbar(mb).colormap insert $colorbar(count) \
		radiobutton -label "$colorbar(map)" \
		-variable colorbar(map) -command "ChangeColormapID $id"
	}
	incr colorbar(count)

	ChangeColormapID $id
    }
}

proc SaveColormap {} {
    global ds9
    global current

    FileLastFull colorbarfbox [colorbar get file name]
    set filename [SaveFileDialog colorbarfbox]
    if {$filename != {}} {
	colorbar save "\{$filename\}"
    }
}

proc LoadContrastBias {} {
    global colorbar
    global colorbardlg

    set filename [OpenFileDialog contrastbiasfbox]
    if {$filename != {}} {
	if {![catch {set ch [open $filename r]}]} {
	    set l [gets $ch]
	    close $ch
	    set colorbardlg(constrast) [lindex $l 0]
	    set colorbardlg(bias) [lindex $l 1]

	    BeginAdjustColormap
	    AdjustColormap
	    EndAdjustColormap
	}
    }
}

proc SaveContrastBias {} {
    global colorbar
    global colorbardlg

    set filename [SaveFileDialog contrastbiasfbox]
    if {$filename != {}} {
	if {![catch {set ch [open $filename w]}]} {
	    puts $ch "$colorbardlg(contrast) $colorbardlg(bias)"
	    close $ch
	}
    }
}

proc ButtonColormap {x y} {
    global colorbar
    global current
    global rgb
    global cursor

    # turn off blinking cursor
    if {$cursor(timer)} {
	catch {after cancel $cursor(id)}
	set cursor(id) 0
    }

    if {$current(frame) != {}} {
	# we need to hold the current frame, since we may be blinking
	set colorbar(frame) $current(frame)
	$colorbar(frame) colormap begin
    }
}

proc MotionColormap {x y} {
    global colorbar
    global current
    global canvas

    # X sets bias
    set bias [expr double($x)/$canvas(width)]

    # Y sets contrast
    set contrast [expr double($y)/$canvas(height) * 10]

    if {$colorbar(frame) != {}} {
	RGBEvalLockColorbar "$current(colorbar) adjust $contrast $bias"
	# only update the current colorbar frame
	$colorbar(frame) colormap motion [$current(colorbar) get colormap]
    } else {
	RGBEvalLockColorbar "$current(colorbar) adjust $contrast $bias"
    }

    UpdateColorDialog
}

proc ReleaseColormap {x y} {
    global colorbar
    global current
    global rgb
    global cursor

    # and turn on blinking cursor if needed
    if {$cursor(timer)} {
	CursorTimer
    }

    # only update the current colorbar frame
    if {$colorbar(frame) != {}} {
	$colorbar(frame) colormap end
	set colorbar(frame) {}
    }

    UpdateColorDialog
}

proc ChangeColormapID {id} {
    global current
    global colorbar
    
    colorbar map $id
    if {$current(frame) != {} } {
	$current(frame) colormap [colorbar get colormap]
	set colorbar(map) [colorbar get name]
	set colorbar(invert) [colorbar get invert]
    }

    UpdateColorDialog
}

proc MatchColorbars {} {
    global ds9
    global current
    global colorbar

    set id [$current(colorbar) get colormap]
    foreach f $ds9(frames) {
	if {$f!=$current(frame) && [$f get type]==[$current(frame) get type]} {
	    $f colormap $id
	}
    }
}

proc InvertColorbar {} {
    global current
    global colorbar

    $current(colorbar) invert $colorbar(invert)
    if {$current(frame) != {} } {
	$current(frame) colormap [$current(colorbar) get colormap]
    }

    UpdateColorDialog
}

proc UpdateColormapLevel {} {
    global ds9
    global current
    global colorbar

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateColormapLevel"
    }

    if {$current(frame) != {}} {
	$current(colorbar) set colormap level \
	    [$current(frame) get colorscale level $colorbar(num)]
    }
}

proc UpdateColormapLevelMosaic {which x y sys} {
    global ds9
    global current
    global colorbar
    global current
    global scale

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateColormapLevelMosaic"
    }

    if {$current(frame) == {}} {
	return
    }

    if {($current(frame) == $which) && 
	($scale(scope) == "local") &&
	[$which has fits mosaic]} {

	set ext [$which get fits ext $sys $x $y]

	if {$current(ext) != $ext} {
	    $current(colorbar) set colormap level \
	       [$current(frame) get colorscale level $colorbar(num) $sys $x $y]
	}

	set current(ext) $ext
    } else {
	set current(ext) {}
    }
}

proc ColormapDialog {} {
    global colorbar
    global colorbardlg
    global ds9

    # see if we already have a window visible

    if [winfo exists $colorbar(top)] {
	raise $colorbar(top)
	return
    }

    set w $colorbar(top)
    set title [msgcat::mc {Colormap Parameters}]
    set length 300

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ColormapDestroyDialog
    wm minsize $w 375 100

    $w configure -menu $colorbar(mb)

    menu $colorbar(mb)

    AppleMenu colorbar(mb)

    $colorbar(mb) add cascade -label [msgcat::mc {File}] \
	-menu $colorbar(mb).file
    $colorbar(mb) add cascade -label [msgcat::mc {Colormap}] \
	-menu $colorbar(mb).colormap

    menu $colorbar(mb).file
    $colorbar(mb).file add command -label [msgcat::mc {Apply}] \
	-command ColormapApplyDialog
    $colorbar(mb).file add separator
    $colorbar(mb).file add command -label "[msgcat::mc {Load Colormap}]..." \
	-command LoadColormap
    $colorbar(mb).file add command -label "[msgcat::mc {Save Colormap}]..." \
	-command SaveColormap
    $colorbar(mb).file add separator
    $colorbar(mb).file add command \
	-label "[msgcat::mc {Load Contrast/Bias}]..."\
	-command LoadContrastBias
    $colorbar(mb).file add command \
	-label "[msgcat::mc {Save Contrast/Bias}]..." \
	-command SaveContrastBias
    $colorbar(mb).file add separator
    $colorbar(mb).file add command -label [msgcat::mc {Close}] \
	-command ColormapDestroyDialog

    menu $colorbar(mb).colormap
    set count 1
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	$colorbar(mb).colormap insert $count radiobutton \
	    -label [msgcat::mc $title] -variable colorbar(map) \
	    -command "ChangeColormapID $ii"
	incr count
    }
    $colorbar(mb).colormap add separator
    $colorbar(mb).colormap add checkbutton \
	-label [msgcat::mc {Invert Colormap}] \
	-variable colorbar(invert) -command InvertColorbar
    $colorbar(mb).colormap add command -label [msgcat::mc {Reset Colormap}] \
	-command ResetColormap

    UpdateColorDialog

    frame $w.cb -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.cb $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.cb.contrast
    scale $w.cb.contrast.slider -from 0 -to 10 -length $length \
	-variable colorbardlg(contrast) -orient horizontal \
	-label [msgcat::mc {Contrast}] \
	-tickinterval 2 -showvalue false -resolution .01 \
	-command AdjustColormap
    entry $w.cb.contrast.value -textvariable colorbardlg(contrast) -width 7
    pack $w.cb.contrast.slider $w.cb.contrast.value -side left -padx 4
    pack $w.cb.contrast.value

    frame $w.cb.bias
    scale $w.cb.bias.slider -from 0 -to 1 -length $length \
	-variable colorbardlg(bias) -orient horizontal \
	-label [msgcat::mc {Bias}] \
	-tickinterval .2 -showvalue false -resolution .001 \
	-command AdjustColormap
    entry $w.cb.bias.value -textvariable colorbardlg(bias) -width 7
    pack $w.cb.bias.slider $w.cb.bias.value -side left -padx 4
    pack $w.cb.bias.value

    pack $w.cb.contrast $w.cb.bias -side top

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command ColormapApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command ColormapDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w.cb.contrast.slider <Button-1> BeginAdjustColormap
    bind $w.cb.contrast.slider <ButtonRelease-1> EndAdjustColormap
    bind $w.cb.bias.slider <Button-1> BeginAdjustColormap
    bind $w.cb.bias.slider <ButtonRelease-1> EndAdjustColormap
}

proc ColormapDestroyDialog {} {
    global colorbar
    global colorbardlg

    if {[winfo exists $colorbar(top)]} {
	destroy $colorbar(top)
	destroy $colorbar(mb)
    }

    if {[info exists colorbardlg]} {
	unset colorbardlg
    }
}

proc ColormapApplyDialog {} {
    BeginAdjustColormap
    AdjustColormap
    EndAdjustColormap
}

proc BeginAdjustColormap {} {
    global current
    global colorbar
    global rgb

    set colorbar(adjustok) 1
    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap begin"
    }
}

proc AdjustColormap {{value {}}} {
    # value is ignored
    global current
    global colorbar
    global colorbardlg
    global rgb

    if {[info exists colorbar(adjustok)]} {
	if {$current(frame) != {}} {
	    RGBEvalLockColorbar "$current(colorbar) adjust $colorbardlg(contrast) $colorbardlg(bias)"
	    RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap motion [$current(colorbar) get colormap]"
	}
    }
}

proc EndAdjustColormap {} {
    global current
    global colorbar
    global rgb

    if {[info exists colorbar(adjustok)]} {
	unset colorbar(adjustok)
	if {$current(frame) != {}} {
	    RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap end"
	}
    }
}

proc UpdateColorDialog {} {
    global current
    global colorbar
    global colorbardlg

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateColorDialog"
    }

    if {[winfo exists $colorbar(top)]} {
	set colorbardlg(contrast) [$current(colorbar) get contrast]
	set colorbardlg(bias) [$current(colorbar) get bias]

	if {$current(frame) != {}} {
	    switch -- [$current(frame) get type] {
		base -
		3d {
		    $colorbar(mb).file entryconfig \
			"[msgcat::mc {Load Colormap}]..." -state normal
		    $colorbar(mb).file entryconfig \
			"[msgcat::mc {Save Colormap}]..." -state normal
		    for {set i $colorbar(start)} {$i<$colorbar(count)} {incr i} {
			$colorbar(mb).colormap entryconfig $i -state normal
		    }
		}
		rgb {
		    $colorbar(mb).file entryconfig \
			"[msgcat::mc {Load Colormap}]..." -state disabled
		    $colorbar(mb).file entryconfig \
			"[msgcat::mc {Save Colormap}]..." -state disabled
		    for {set i $colorbar(start)} {$i<$colorbar(count)} {incr i} {
			$colorbar(mb).colormap entryconfig $i -state disabled
		    }
		}
	    }
	} else {
	    $colorbar(mb).file entryconfig \
		"[msgcat::mc {Load Colormap}]..." -state normal
	    $colorbar(mb).file entryconfig \
		"[msgcat::mc {Save Colormap}]..." -state normal
	    for {set i $colorbar(start)} {$i<$colorbar(count)} {incr i} {
		$colorbar(mb).colormap entryconfig $i -state normal
	    }
	}
    }
}

proc LayoutColorbar {} {
    global ds9
    global canvas
    global colorbar

    switch -- $colorbar(orientation) {
	horizontal {
	    set xx 0
	    set yy [expr $canvas(height) + $canvas(gap)]

	    set colorbar(width) $canvas(width)
	    set colorbar(height) $colorbar(horizontal,height)

	    switch -- $ds9(visual) {
		pseudocolor {
		    colorbar configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 0 \
			-anchor nw \
			-numopts [ColorbarOpts]
		}
		truecolor {
		    colorbar configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 0 \
			-anchor nw \
			-numopts [ColorbarOpts]
		    colorbarrgb configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 0 \
			-anchor nw \
			-numopts [ColorbarOpts]
		}
	    }
	}
	vertical {
	    set xx [expr $canvas(width) + $canvas(gap)]
	    set yy 0

	    set colorbar(width) $colorbar(vertical,width)
	    set colorbar(height) $canvas(height)

	    switch -- $ds9(visual) {
		pseudocolor {
		    colorbar configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 1 \
			-anchor nw \
			-numopts [ColorbarOpts]
		}
		truecolor {
		    colorbar configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 1 \
			-anchor nw \
			-numopts [ColorbarOpts]
		    colorbarrgb configure -x $xx -y $yy \
			-width $colorbar(width) \
			-height $colorbar(height) \
			-orientation 1 \
			-anchor nw \
			-numopts [ColorbarOpts]
		}
	    }
	}
    }
}

proc ColorbarOpts {} {
    global colorbar

    set rr "Grid=0, DrawAxes=0, MinTickLen(1)=0, MinTickLen(2)=0, Colour(ticks)=0, Width(ticks)=1, Style(ticks)=0, Border=1, Colour(border)=0, Width(border)=0, Style(border)=0, Labelling=exterior, LabelUp=0, Colour(numlab)=0, TextLab=0, DrawTitle=0"

    switch -- $colorbar(orientation) {
	horizontal {
	    append rr ", Edge(1)=top, Edge(2)=left, NumLabGap=$colorbar(horizontal,gap)"
	}
	vertical {
	    append rr ", Edge(1)=bottom, Edge(2)=right, NumLabGap=$colorbar(vertical,gap)"
	}
    }

    append rr ", Font(numlab)=[GridFont2Ast $colorbar(font) $colorbar(font,style)], Size(numlab)=$colorbar(font,size)" 

    if {$colorbar(numerics)} {
	switch -- $colorbar(orientation) {
	    horizontal {
		append rr ", MajTickLen(1)=-.15, MajTickLen(2)=0, NumLab(1)=1, NumLab(2)=0"
	    }
	    vertical {
		append rr ", MajTickLen(1)=0, MajTickLen(2)=-.15, NumLab(1)=0, NumLab(2)=1"
	    }
	}
	
    } else {
	append rr ", NumLab=0, MajTickLen=0"
    }

    return $rr
}

# Process Cmds

proc ProcessCmapCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global current
    global colorbar
    global rgb

    switch -- [string tolower [lindex $var $i]] {
	file {
	    incr i
	    set fn [lindex $var $i]
	    LoadColormapFile $fn
	    FileLast colormapfbox $fn
	}
	invert {
	    incr i
	    set colorbar(invert) [FromYesNo [lindex $var $i]]
	    InvertColorbar
	}
	value {
	    # we need to be realized
	    RealizeDS9

	    incr i
	    set c [lindex $var $i]
	    incr i
	    set b [lindex $var $i]
	    if {$current(frame) != {}} {
		RGBEvalLockColorbar "$current(colorbar) adjust $c $b"
		RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap begin"
		RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap motion [$current(colorbar) get colormap]"
		RGBEvalLock rgb(lock,colorbar) "$current(frame) colormap end"
	    }
	}
	default {
	    switch -- [$current(frame) get type] {
		base -
		3d {
		    set cmap [lindex $var $i]
		    # common variants on spellings
		    switch -- [string tolower $cmap] {
			gray {set cmap grey}
		    }

		    set id [colorbar list id]
		    set found 0
		    foreach ii $id {
			set title [colorbar get name $ii]
			if {![string compare -nocase $title $cmap]} {
			    set colorbar(map) $title
			    colorbar map "{$colorbar(map)}"
			    $current(frame) colormap [colorbar get colormap]
			    set colorbar(invert) [colorbar get invert]

			    set found 1
			    break
			}
		    }
		    if {!$found} {
			Error "[msgcat::mc {Unknown Colormap}] $cmap"
		    }
		}
		rgb {}
	    }
	}
    }

    UpdateColorDialog
}

proc ProcessSendCmapCmd {proc id param} {
    global current
    global colorbar

    switch -- [string tolower $param] {
	file {$proc $id "[$current(colorbar) get file name]\n"}
	invert {$proc $id [ToYesNo $colorbar(invert)]}
	value {$proc $id "[$current(colorbar) get contrast] [$current(colorbar) get bias]\n"}
	{} {$proc $id "[$current(colorbar) get name]\n"}
    }
}

proc ProcessColorbarCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global colorbar
    global view

    set item [string tolower [lindex $var $i]]

    switch  -- $item {
	numerics {
	    incr i
	    set yesno [string tolower [lindex $var $i]]
	    set colorbar(numerics) [FromYesNo $yesno]
	    UpdateView
	}
	font {
	    incr i
	    set item [string tolower [lindex $var $i]]
	    set colorbar(font) $item
	    UpdateView
	}
	fontsize {
	    incr i
	    set item [lindex $var $i]
	    set colorbar(font,size) $item
	    UpdateView
	}
	fontstyle {
	    incr i
	    set item [string tolower [lindex $var $i]]
	    set colorbar(font,style) $item
	    UpdateView
	}
	orientation {
	    incr i
	    set item [string tolower [lindex $var $i]]
	    set colorbar(orientation) $item
	    UpdateView
	}
	vertical -
	horizontal {
	    set colorbar(orientation) $item
	    UpdateView
	}
	default {
	    set yesno [string tolower [lindex $var $i]]
	    set view(colorbar) [FromYesNo $yesno]
	    UpdateView
	}
    }
}

proc ProcessSendColorbarCmd {proc id param} {
    global colorbar
    global view

    switch -- [string tolower [lindex $param 0]] {
	orientation {$proc $id "$colorbar(orientation)\n"} 
	numerics {$proc $id [ToYesNo $colorbar(numerics)]} 
	font {$proc $id "$colorbar(font)\n"} 
	fontsize {$proc $id "$colorbar(font,size)\n"} 
	fontstyle {$proc $id "$colorbar(font,style)\n"} 
	default {$proc $id [ToYesNo $view(colorbar)]} 
    }
}
