#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc BadVisualError {} {
    global ds9

    Error [msgcat::mc {Sorry, DS9 requires a Pseudocolor8, Truecolor8, Truecolor16, Truecolor24 visual be available}]
    exit
}

proc ProcessCommandLineFirst {} {
    global argc
    global argv
    global colorbar
    global ds9

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]

	switch -- $item {
	    -help {
		puts "For more information, use --help"
		QuitDS9
	    }
	    -debug {incr i; ProcessDebugTclCmd argv i}
	    -private {set colorbar(private) true}
	    -title {
		incr i
		set ds9(title) [lindex $argv $i]
		set t "SAOImage $ds9(title)"
		wm title $ds9(top) "$t"
		wm iconname $ds9(top) "$t"
	    }
	    -language {
		incr i
		set ds9(language) [lindex $argv $i]
		set ds9(language,name) [LanguageToName $ds9(language)]
	    }
	    -msg {
		incr i
		set ds9(language,dir) [lindex $argv $i]
	    }
	}
	incr i
    }
}

proc ProcessCommandLineXPA {} {
    global argc
    global argv

    set i 0
    while {$i < $argc} {
	set item [lindex $argv $i]
	switch -- $item {
	    -vo {incr i; ProcessVOCmd argv i}
	}
	incr i
    }
}

proc ProcessCommandLine {} {
    global argc
    global argv

    ProcessCommand $argv $argc
}

proc ds9Cmd {argv} {
    ProcessCommand $argv [llength $argv]
}

proc ProcessCommand {argv argc} {
    global ds9
    global help
    global current
    global contour
    global colorbar
    global debug
    global wcs
    global view
    global grid
    global bin
    global scale
    global minmax
    global zscale
    global iis
    global http

    set load 0
    set file(mode) fits
    set file(layer) {}
    set file(mode,mosaic) wcs
    set i 0

    # Note: -help is processed previously for fv (temporary)
    set item {}

    while {$i < $argc} {
	set item [lindex $argv $i]

	switch -- $item {
	    -? -
	    -help -
	    --help {HelpCommand}

	    -version {
		ProcessSendVersionCmd puts stdout {}
		QuitDS9
	    }

	    -2mass {incr i; Process2MASSCmd argv i}
	    -3d {incr i; Process3DCmd argv i}
	    -about {ProcessSendAboutCmd puts stdout {} {} {}}
	    -align {
		# backward compatibility
		set wcs(align) 1
		AlignWCSFrame
	    }
	    -analysis -
	    -analysismenu {incr i; ProcessAnalysisCmd argv i {} {}}
	    -array {set file(mode) array}
	    -bin {incr i; ProcessBinCmd argv i}
	    -blink {incr i; ProcessBlinkCmd argv i}
	    -blue {set current(rgb) blue; RGBChannel}
	    -cat -
	    -catalog {incr i; ProcessCatalogCmd argv i}
	    -cd {incr i; ProcessCDCmd argv i}
	    -cmap {incr i; ProcessCmapCmd argv i}
	    -colorbar {incr i; ProcessColorbarCmd argv i}
	    -console {ProcessConsoleCmd argv i}
	    -contour {incr i; ProcessContourCmd argv i}
	    -nocontour {set contour(view) 0; ContourUpdate}
	    -crosshair {incr i; ProcessCrosshairCmd argv i}
	    -cursor {incr i; ProcessCursorCmd argv i}
	    -datacube {incr i; ProcessDataCubeCmd argv i}
	    -debug {incr i; ProcessDebugCmd argv i}
	    -dss -
	    -dsssao -
	    -sao {incr i; ProcessSAOCmd argv i}
	    -dsseso -
	    -eso {incr i; ProcessESOCmd argv i}
	    -dssstsci -
	    -stsci {incr i; ProcessSTSCICmd argv i}
	    -exit -
	    -quit {ProcessQuitCmd argv i}
	    -frame {incr i; ProcessFrameCmd argv i}
	    -fifo {
		incr i
		set fifo [lindex $argv $i]
		if {$fifo!="none"} {
		    set iis(ififo) ${fifo}i
		    set iis(ofifo) ${fifo}o
		} else {
		    set iis(ififo) none
		    set iis(ofifo) none
		}
	    }
	    -fifo_only {
		set iis(port) 0
		set iis(unix) none
	    }
	    -first {incr i; ProcessFIRSTCmd argv i}
	    -file -
	    -fits {set file(mode) fits}
	    -sfits {set file(mode) sfits}
	    -geometry {
		# already processed
	    }
	    -green {set current(rgb) green; RGBChannel}
	    -grid {incr i; ProcessGridCmd argv i}
	    -nogrid {set grid(view) 0; GridUpdate}
	    -header {incr i; ProcessHeaderCmd argv i}
	    -height {incr i; ProcessHeightCmd argv i}
	    -histequ {set scale(type) histequ; ChangeScale}
	    -horzgraph {
		# backward compatibility
		set view(graph,horz) 1
		UpdateView
	    }
	    -nohorzgraph {
		# backward compatibility
		set view(graph,horz) 0
		UpdateView
	    }
	    -iconify {incr i; ProcessIconifyCmd argv i}
	    -iis {incr i; ProcessIISCmd argv i}
	    -info {
		# backward compatibility
		set view(info) 1
		UpdateView
	    }
	    -noinfo {
		# backward compatibility
		set view(info) 0
		UpdateView
	    }
	    -invert {set colorbar(invert) 1; InvertColorbar}
	    -language {
		# already processed
		incr i
	    }
	    -lock {incr i; ProcessLockCmd argv i}
	    -linear {set scale(type) linear; ChangeScale}
	    -log {set scale(type) log; ChangeScale}
	    -lower {ProcessLowerCmd argv i}
	    -magnifier {
		# backward compatibility
		set view(magnifier) 1
		UpdateView
	    }
	    -nomagnifier {
		# backward compatibility
		set view(magnifier) 0
		UpdateView
	    }
	    -mask {incr i; set file(layer) [ProcessMaskCmd argv i]}
	    -match {incr i; ProcessMatchCmd argv i}
	    -medatacube {set file(mode) medatacube}
	    -minmax {incr i; ProcessMinMaxCmd argv i}
	    -minmaxmode {
		# backward compatibility
		incr i
		set minmax(mode) [lindex $argv $i]
	    }
	    -minmaxsample {
		# backward compatibility
		incr i
		set minmax(sample) [lindex $argv $i]
	    }
	    -mode {incr i; ProcessModeCmd argv i}

	    -mosaicimage {
		set file(mode) mosaicimage
		set aa [lindex $argv [expr $i+1]]
		switch -- $aa {
		    iraf {incr i; set file(mode,mosaic) iraf}
		    wfpc2 {incr i; set file(mode,mosaic) wfpc2}
		    wcs {incr i; set file(mode,mosaic) wcs}
		    wcsa {incr i; set file(mode,mosaic) wcsa}
		    wcsb {incr i; set file(mode,mosaic) wcsb}
		    wcsc {incr i; set file(mode,mosaic) wcsc}
		    wcsd {incr i; set file(mode,mosaic) wcsd}
		    wcse {incr i; set file(mode,mosaic) wcse}
		    wcsf {incr i; set file(mode,mosaic) wcsf}
		    wcsg {incr i; set file(mode,mosaic) wcsg}
		    wcsh {incr i; set file(mode,mosaic) wcsh}
		    wcsi {incr i; set file(mode,mosaic) wcsi}
		    wcsj {incr i; set file(mode,mosaic) wcsj}
		    wcsk {incr i; set file(mode,mosaic) wcsk}
		    wcsl {incr i; set file(mode,mosaic) wcsl}
		    wcsm {incr i; set file(mode,mosaic) wcsm}
		    wcsn {incr i; set file(mode,mosaic) wcsn}
		    wcso {incr i; set file(mode,mosaic) wcso}
		    wcsp {incr i; set file(mode,mosaic) wcsp}
		    wcsq {incr i; set file(mode,mosaic) wcsq}
		    wcsr {incr i; set file(mode,mosaic) wcsr}
		    wcss {incr i; set file(mode,mosaic) wcss}
		    wcst {incr i; set file(mode,mosaic) wcst}
		    wcsu {incr i; set file(mode,mosaic) wcsu}
		    wcsv {incr i; set file(mode,mosaic) wcsv}
		    wcsw {incr i; set file(mode,mosaic) wcsw}
		    wcsx {incr i; set file(mode,mosaic) wcsx}
		    wcsy {incr i; set file(mode,mosaic) wcsy}
		    wcsz {incr i; set file(mode,mosaic) wcsz}
		    default {set file(mode,mosaic) wcs}
		}
	    }
	    -mosaic {
		set file(mode) mosaic
		set aa [lindex $argv [expr $i+1]]
		switch -- $aa {
		    iraf {incr i; set file(mode,mosaic) iraf}
		    wcs {incr i; set file(mode,mosaic) wcs}
		    wcsa {incr i; set file(mode,mosaic) wcsa}
		    wcsb {incr i; set file(mode,mosaic) wcsb}
		    wcsc {incr i; set file(mode,mosaic) wcsc}
		    wcsd {incr i; set file(mode,mosaic) wcsd}
		    wcse {incr i; set file(mode,mosaic) wcse}
		    wcsf {incr i; set file(mode,mosaic) wcsf}
		    wcsg {incr i; set file(mode,mosaic) wcsg}
		    wcsh {incr i; set file(mode,mosaic) wcsh}
		    wcsi {incr i; set file(mode,mosaic) wcsi}
		    wcsj {incr i; set file(mode,mosaic) wcsj}
		    wcsk {incr i; set file(mode,mosaic) wcsk}
		    wcsl {incr i; set file(mode,mosaic) wcsl}
		    wcsm {incr i; set file(mode,mosaic) wcsm}
		    wcsn {incr i; set file(mode,mosaic) wcsn}
		    wcso {incr i; set file(mode,mosaic) wcso}
		    wcsp {incr i; set file(mode,mosaic) wcsp}
		    wcsq {incr i; set file(mode,mosaic) wcsq}
		    wcsr {incr i; set file(mode,mosaic) wcsr}
		    wcss {incr i; set file(mode,mosaic) wcss}
		    wcst {incr i; set file(mode,mosaic) wcst}
		    wcsu {incr i; set file(mode,mosaic) wcsu}
		    wcsv {incr i; set file(mode,mosaic) wcsv}
		    wcsw {incr i; set file(mode,mosaic) wcsw}
		    wcsx {incr i; set file(mode,mosaic) wcsx}
		    wcsy {incr i; set file(mode,mosaic) wcsy}
		    wcsz {incr i; set file(mode,mosaic) wcsz}
		    default {set file(mode,mosaic) wcs}
		}
	    }
	    -smosaic {
		set file(mode) smosaic
		set aa [lindex $argv [expr $i+1]]
		switch -- $aa {
		    iraf {incr i; set file(mode,mosaic) iraf}
		    wcs {incr i; set file(mode,mosaic) wcs}
		    wcsa {incr i; set file(mode,mosaic) wcsa}
		    wcsb {incr i; set file(mode,mosaic) wcsb}
		    wcsc {incr i; set file(mode,mosaic) wcsc}
		    wcsd {incr i; set file(mode,mosaic) wcsd}
		    wcse {incr i; set file(mode,mosaic) wcse}
		    wcsf {incr i; set file(mode,mosaic) wcsf}
		    wcsg {incr i; set file(mode,mosaic) wcsg}
		    wcsh {incr i; set file(mode,mosaic) wcsh}
		    wcsi {incr i; set file(mode,mosaic) wcsi}
		    wcsj {incr i; set file(mode,mosaic) wcsj}
		    wcsk {incr i; set file(mode,mosaic) wcsk}
		    wcsl {incr i; set file(mode,mosaic) wcsl}
		    wcsm {incr i; set file(mode,mosaic) wcsm}
		    wcsn {incr i; set file(mode,mosaic) wcsn}
		    wcso {incr i; set file(mode,mosaic) wcso}
		    wcsp {incr i; set file(mode,mosaic) wcsp}
		    wcsq {incr i; set file(mode,mosaic) wcsq}
		    wcsr {incr i; set file(mode,mosaic) wcsr}
		    wcss {incr i; set file(mode,mosaic) wcss}
		    wcst {incr i; set file(mode,mosaic) wcst}
		    wcsu {incr i; set file(mode,mosaic) wcsu}
		    wcsv {incr i; set file(mode,mosaic) wcsv}
		    wcsw {incr i; set file(mode,mosaic) wcsw}
		    wcsx {incr i; set file(mode,mosaic) wcsx}
		    wcsy {incr i; set file(mode,mosaic) wcsy}
		    wcsz {incr i; set file(mode,mosaic) wcsz}
		    default {set file(mode,mosaic) wcs}
		}
	    }

	    -mosaicimageiraf {
		# backward compatibility
		set file(mode) mosaicimage
		set file(mode,mosaic) iraf
	    }
	    -mosaicimagewcs {
		# backward compatibility
		set file(mode) mosaicimage
		set file(mode,mosaic) wcs
	    }
	    -mosaicimagewfpc2 {
		# backward compatibility
		set file(mode) mosaicimage
		set file(mode,mosaic) wfpc2
	    }

	    -mosaiciraf {
		# backward compatibility
		set file(mode) mosaic
		set file(mode,mosaic) iraf
	    }
	    -mosaicwcs {
		# backward compatibility
		set file(mode) mosaic
		set file(mode,mosaic) wcs
	    }

	    -multiframe {set file(mode) multiframe}

	    -msg {
		# already processed
		incr i
	    }
	    -nameserver {incr i; ProcessNRESCmd argv i}
	    -nvss {incr i; ProcessNVSSCmd argv i}
	    -orient {incr i; ProcessOrientCmd argv i}
	    -pagesetup {incr i; ProcessPageSetupCmd argv i}
	    -pan {RealizeDS9; incr i; ProcessPanCmd argv i}
	    -panner {
		# backward compatibility
		set view(panner) 1
		UpdateView
	    }
	    -nopanner {
		# backward compatibility
		set view(panner) 0
		UpdateView
	    }
	    -pixeltable {incr i; ProcessPixelTableCmd argv i}
	    -nopixeltable {PixelTableDestroyDialog}
	    -plot {incr i; ProcessPlotCmd argv i {} {}}
	    -port {incr i; set iis(port) [lindex $argv $i]}
	    -inet_only -
	    -port_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(unix) none
	    }
	    -pow {set scale(type) pow; ChangeScale}
	    -prefs {incr i; ProcessPrefsCmd argv i}
	    -preserve {incr i; ProcessPreserveCmd argv i}
	    -print {incr i; ProcessPrintCmd argv i}
	    -private {
		#already processed
	    }
	    -raise {ProcessRaiseCmd argv i}
	    -red {set current(rgb) red; RGBChannel}
	    -region -
	    -regions -
	    -regionfile {RealizeDS9; incr i; ProcessRegionsCmd argv i {} {}}
	    -rgb {incr i; ProcessRGBCmd argv i}
	    -rgbcube {set file(mode) rgbcube}
	    -srgbcube {set file(mode) srgbcube}
	    -rgbimage {set file(mode) rgbimage}
	    -rgbarray {set file(mode) rgbarray}
	    -rotate {incr i; ProcessRotateCmd argv i}
	    -samp {incr i; ProcessSAMPCmd argv i}
	    -saveas -
	    -saveimage {incr i; ProcessSaveImageCmd argv i}
	    -savefits {incr i; ProcessSaveFitsCmd argv i}
	    -savempeg {incr i; ProcessSaveMPEGCmd argv i}
	    -scale -
	    -ztrans {incr i; ProcessScaleCmd argv i}
	    -scalelims -
	    -scalelimits {
		#backward compatibility
		incr i
		set scale(min) [lindex $argv $i]
		incr i
		set scale(max) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -scalemode {
		#backward compatibility
		incr i
		set scale(mode) [string tolower [lindex $argv $i]]
		ChangeScaleMode
	    }
	    -scalescope {
		#backward compatibility
		incr i
		set scale(scope) [string tolower [lindex $argv $i]]
		ChangeScaleScope
	    }
	    -shm {incr i; ProcessShmCmd argv i 1}
	    -single {ProcessSingleCmd argv i}
	    -skyview {incr i; ProcessSkyViewCmd argv i}
	    -sleep {incr i; ProcessSleepCmd argv i}
	    -smooth {incr i; ProcessSmoothCmd argv i}
	    -squared {set scale(type) squared; ChangeScale}
	    -sqrt {set scale(type) sqrt; ChangeScale}
	    -source {incr i; ProcessSourceCmd argv i}
	    -tcl {incr i; set ds9(tcl) [FromYesNo [lindex $argv $i]]}
	    -tile {incr i; ProcessTileCmd argv i}
	    -title {
		#already processed
		incr i
	    }
	    -unix {incr i; set iis(unix) [lindex $argv $i]}
	    -unix_only {
		set iis(ififo) none
		set iis(ofifo) none
		set iis(port) 0
	    }
	    -url {set file(mode) url}
	    -update {incr i; ProcessUpdateCmd argv i}
	    -vertgraph {
		#backward compatibility
		set view(graph,vert) 1
		UpdateView
	    }
	    -novertgraph {
		#backward compatibility
		set view(graph,vert) 0
		UpdateView
	    }
	    -coords -
	    -view {incr i; ProcessViewCmd argv i}
	    -visual {
		#already processed
	    }
	    -vo {incr i; ProcessVOCmd argv i}
	    -wcs {incr i; ProcessWCSCmd argv i {} {}}
	    -wcsformat {
		#backward compatibility
		incr i
		set wcs(format,) [lindex $argv $i]
	    }
	    -web {incr i; ProcessWebCmd argv i}
	    -width {incr i; ProcessWidthCmd argv i}
	    -xpa {incr i; ProcessXPACmd argv i}
	    -z1 {
		#backward compatibility
		incr i
		set scale(min) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -z2 {
		#backward compatibility
		incr i
		set scale(max) [lindex $argv $i]
		ChangeScaleLimit
	    }
	    -zscale {incr i; ProcessZScaleCmd argv i}
	    -zmax {set scale(mode) zmax; ChangeScaleMode}
	    -zoom {incr i; ProcessZoomCmd argv i}

	    default {
		# allow abc, -, and -[foo] but not -abc
		if {![regexp -- {^-[^[]+} $item]} {
		    if {$load == 0} {
			StartLoad
			incr load
		    }

		    if {$current(frame) == {}} {
			return
		    }

		    switch $ds9(wm) {
			x11 -
			aqua {CommandLineLoad $item file argv i}
			win32 {
			    # if win32 and envoked via DOS shell
			    # we must expand wildcards ourselves
			    if {[catch {glob $item} fns]} {
				# cygwin/double click/DOS Shell no wildcards
				CommandLineLoad $item file argv i
			    } else {
				# DOS Shell with wildcards
				foreach fn $fns {
				    CommandLineLoad $fn file argv i
				}
			    }
			}
		    }
		} else {
		    puts stderr "[msgcat::mc {Unknown command}]: $item"
		    puts stderr "[msgcat::mc {For more information, use --help}]"
		    return
		}
	    }
	}
	incr i
    }

    if {$load != 0} {
	FinishLoad
    }
}

proc CommandLineLoad {item filename argvname iname} {
    upvar $filename file
    upvar $argvname argv
    upvar $iname i

    global current

    switch -- [$current(frame) get type] {
	base {CommandLineLoadBase $item $filename $argvname $iname}
	rgb {CommandLineLoadRGB $item $filename $argvname $iname}
	3d {CommandLineLoad3D $item $filename $argvname $iname}
    }

    switch -- $file(mode) {
	array {FileLast arrayfbox $item}
	url {}
	default {FileLast fitsfbox $item}
    }

    GridUpdate
    ContourUpdate
}

proc CommandLineLoadBase {item filename argvname iname} {
    upvar 2 $filename file
    upvar 2 $argvname argv
    upvar 2 $iname i

    global ds9

    switch -- $file(mode) {
	array {
	    MultiLoad $file(layer)
	    LoadArray $item $file(layer)
	}
	fits {
	    # under windows, a double click on a 
	    # data file comes here
	    switch $ds9(wm) {
		x11 {}
		aqua {}
		win32 {
		    # allow abc but not -abc, -, and -[foo]
		    if {![regexp -- {^-} $item]} {
			set item [file normalize [file nativename $item]]
		    }				
		}
	    }
	    MultiLoad $file(layer)
	    LoadFits $item $file(layer)
	}
	sfits {
	    MultiLoad $file(layer)
	    incr i
	    LoadSFits $item [lindex $argv $i] $file(layer)
	}
	medatacube {
	    MultiLoad $file(layer)
	    LoadDataCubeFits $item $file(layer)
	}
	multiframe {
	    MultiLoad
	    LoadMultiFrameFits $item
	}
	mosaicimage {
	    MultiLoad $file(layer)
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicImageIRAFFits $item $file(layer)}
		wfpc2 {LoadMosaicImageWFPC2Fits $item}
		default {LoadMosaicImageWCSFits $file(mode,mosaic) \
			     $item $file(layer)}
	    }
	}
	mosaic {
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFFits $item $file(layer)}
		default {LoadMosaicWCSFits $file(mode,mosaic) \
			     $item $file(layer)}
	    }
	}
	smosaic {
	    incr i
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFSFits $item [lindex $argv $i] $file(layer)}
		default {LoadMosaicWCSSFits $file(mode,mosaic) \
			     $item [lindex $argv $i] $file(layer)}
	    }
	}

	rgbcube {
	    CreateRGBFrame
	    LoadRGBCubeFits $item
	}
	srgbcube {
	    CreateRGBFrame
	    incr i
	    LoadRGBCubeSFits $item [lindex $argv $i]
	}
	rgbimage {
	    CreateRGBFrame
	    LoadRGBImageFits $item
	}
	rgbarray {
	    CreateRGBFrame
	    LoadRGBArray $item
	}

	url {
	    MultiLoad $file(layer)
	    LoadURL $item
	}
    }
}

proc CommandLineLoadRGB {item filename argvname iname} {
    upvar 2 $filename file
    upvar 2 $argvname argv
    upvar 2 $iname i

    switch -- $file(mode) {
	array {LoadArray $item}
	fits {LoadFits $item}
	sfits {
	    incr i
	    LoadSFits $item [lindex $argv $i]
	}
	medatacube {LoadDataCubeFits $item}
	multiframe {
	    MultiLoad
	    LoadMultiFrameFits $item
	}
	mosaicimage {
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicImageIRAFFits $item}
		wfpc2 {LoadMosaicImageWFPC2Fits $item}
		default {LoadMosaicImageWCSFits $file(mode,mosaic) $item}
	    }
	}
	mosaic {
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFFits $item}
		default {LoadMosaicWCSFits $file(mode,mosaic) $item}
	    }
	}
	smosaic {
	    incr i
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFSFits $item [lindex $argv $i]}
		default {LoadMosaicWCSSFits $file(mode,mosaic) \
			     $item [lindex $argv $i]}
	    }
	}

	rgbcube {
	    CreateRGBFrame
	    LoadRGBCubeFits $item
	}
	srgbcube {
	    CreateRGBFrame
	    incr i
	    LoadRGBCubeSFits $item [lindex $argv $i]
	}
	rgbimage {
	    CreateRGBFrame
	    LoadRGBImageFits $item
	}
	rgbarray {
	    CreateRGBFrame
	    LoadRGBArray $item
	}

	url {LoadURL $item}
    }
}

proc CommandLineLoad3D {item filename argvname iname} {
    upvar 2 $filename file
    upvar 2 $argvname argv
    upvar 2 $iname i

    switch -- $file(mode) {
	array {
	    MultiLoad
	    LoadArray $item
	}
	fits {
	    MultiLoad
	    LoadFits $item
	}
	sfits {
	    MultiLoad
	    incr i
	    LoadSFits $item [lindex $argv $i]
	}
	medatacube {
	    MultiLoad
	    LoadDataCubeFits $item
	}
	multiframe {
	    MultiLoad
	    LoadMultiFrameFits $item
	}
	mosaicimage {
	    MultiLoad
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicImageIRAFFits $item}
		wfpc2 {LoadMosaicImageWFPC2Fits $item}
		default {LoadMosaicImageWCSFits $file(mode,mosaic) $item}
	    }
	}
	mosaic {
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFFits $item}
		default {LoadMosaicWCSFits $file(mode,mosaic) $item}
	    }
	}
	smosaic {
	    incr i
	    switch -- $file(mode,mosaic) {
		iraf {LoadMosaicIRAFSFits $item [lindex $argv $i]}
		default {LoadMosaicWCSSFits $file(mode,mosaic) $item \
			     [lindex $argv $i]}
	    }
	}
	url {
	    MultiLoad
	    LoadURL $item
	}
    }
}
