#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBasePandaRectDialog {frame id width height} {
    global marker
    global ds9

    set w ".marker$id"
    set mb ".mb$id"

    # base panda dialog
    MarkerBasePandaDialog $frame $id $width $height
    set which $marker($frame,$id,which)

    # callbacks
    $frame marker $id callback rotate MarkerBaseCenterRotateCB $frame

    # Radius
    label $w.ref.left.bottom.f.majorTitle -text [msgcat::mc {Major}]
    label $w.ref.left.bottom.f.minorTitle -text [msgcat::mc {Minor}]

    label $w.ref.left.bottom.f.outerTitle -text [msgcat::mc {Outer}]
    entry $w.ref.left.bottom.f.radius1 \
	-textvariable marker($frame,$id,radius1) \
	-width 13 
    entry $w.ref.left.bottom.f.radius2 \
	-textvariable marker($frame,$id,radius2) \
	-width 13 
    label $w.ref.left.bottom.f.radiusformat \
	-textvariable marker($frame,$id,dlabel) -padx 4

    label $w.ref.left.bottom.f.innerTitle -text [msgcat::mc {Inner}]
    entry $w.ref.left.bottom.f.radius3 \
	-textvariable marker($frame,$id,radius3) -width 13 

    label $w.ref.left.bottom.f.annuliTitle -text [msgcat::mc {Annuli}]
    entry $w.ref.left.bottom.f.annuli -textvariable marker($frame,$id,annuli) \
	-width 13

    grid x $w.ref.left.bottom.f.majorTitle $w.ref.left.bottom.f.minorTitle \
	-padx 4 -sticky w
    grid $w.ref.left.bottom.f.outerTitle $w.ref.left.bottom.f.radius1 \
	$w.ref.left.bottom.f.radius2 $w.ref.left.bottom.f.radiusformat \
	-padx 4 -sticky w
    grid $w.ref.left.bottom.f.innerTitle $w.ref.left.bottom.f.radius3 \
	-padx 4 -sticky w
    grid $w.ref.left.bottom.f.annuliTitle $w.ref.left.bottom.f.annuli \
	-padx 4 -sticky w

    # Angle
    label $w.ref.left.bottom.f.angleTitle -text [msgcat::mc {Angle}]
    entry $w.ref.left.bottom.f.angle -textvariable marker($frame,$id,angle) \
	-width 13 
    label $w.ref.left.bottom.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $w.ref.left.bottom.f.angleTitle $w.ref.left.bottom.f.angle x \
	$w.ref.left.bottom.f.angleUnit -padx 4 -sticky w

    # init - do this last
    ${which}DistCB $frame $id
    MarkerBaseCenterRotateCB $frame $id
}

# actions

proc MarkerBasePandaRectClose {frame id} {
    MarkerBasePandaClose $frame $id
    $frame marker $id delete callback rotate MarkerBaseCenterRotateCB
}

proc MarkerBasePandaRectApply {frame id} {
    MarkerBasePandaApply $frame $id
    MarkerBaseCenterRotate $frame $id
}

# callbacks

proc MarkerBasePandaRectCoordCB {frame id} {
    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectCoordCB"
    }
    MarkerBasePandaCoordCB $frame $id
    MarkerBaseCenterRotateCB $frame $id
}

proc MarkerBasePandaRectEditCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectEditCB"
    }
    set which $marker($frame,$id,which)

    set t \
	[$frame get marker $id epanda radius \
	     $marker($frame,$id,dcoord) $marker($frame,$id,dformat)]

    set last [llength $t]
    set marker($frame,$id,annuli) [expr $last/2-1]
    set marker($frame,$id,radius1) [lindex $t [expr $last-2]]
    set marker($frame,$id,radius2) [lindex $t [expr $last-1]]
    set marker($frame,$id,radius3) [lindex $t 0]

    $marker($frame,$id,annulitxt) delete 1.0 end
    $marker($frame,$id,annulitxt) insert end "$t"

    set a [$frame get marker $id $which angle \
	       $marker($frame,$id,system) $marker($frame,$id,sky)]

    set last [expr [llength $a]-1]
    set marker($frame,$id,ang1) [lindex $a 0]
    set marker($frame,$id,ang2) [lindex $a $last]
    set marker($frame,$id,angnum) $last

    $marker($frame,$id,angtxt) delete 1.0 end
    $marker($frame,$id,angtxt) insert end "$a"
}

proc MarkerBasePandaRectDistCB {frame id} {
    global debug
    if {$debug(tcl,marker)} {
	puts stderr "MarkerBasePandaRectDistCB"
    }
    MarkerBasePandaDistCB $frame $id
}
