#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MaskDef {} {
    global mask
    global pmask

    set mask(top) .msk
    set mask(mb) .mskmb

    set mask(color) red
    set mask(mark) 1
    set mask(transparency) 0

    array set pmask [array get mask]
    unset pmask(top)
    unset pmask(mb)
}

proc MaskTransparency {dummy} {
    global mask
    global current

    if {$current(frame) != {}} {
	$current(frame) mask transparency $mask(transparency)
    }
}

proc MaskClear {} {
    global current
    global mask

    if {$current(frame) != {}} {
	$current(frame) mask clear
    }
}

proc MaskDialog {} {
    global mask
    global current
    global ds9

    # see if we already have a ctr window visible
    if [winfo exists $mask(top)] {
	raise $mask(top)
	return
    }

    set w $mask(top)
    set title [msgcat::mc {Mask Parameters}]

    # create the mask window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW MaskDestroyDialog
    wm minsize $w 375 100

    $w configure -menu $mask(mb)

    menu $mask(mb)

    AppleMenu mask(mb)

    $mask(mb) add cascade -label [msgcat::mc {File}] \
	-menu $mask(mb).file
    $mask(mb) add cascade -label [msgcat::mc {Edit}] \
	-menu $mask(mb).edit

    menu $mask(mb).file
    $mask(mb).file add command -label "[msgcat::mc {Open}]..." \
	-command [list OpenFits mask]
    $mask(mb).file add cascade -label [msgcat::mc {Open Other}] \
	-menu $mask(mb).file.other
    $mask(mb).file add separator
    $mask(mb).file add command -label [msgcat::mc {Apply}] \
	-command MaskApplyDialog
    $mask(mb).file add command -label [msgcat::mc {Clear}] \
	-command MaskClear
    $mask(mb).file add separator
    $mask(mb).file add command -label [msgcat::mc {Close}] \
	-command MaskDestroyDialog

    menu $mask(mb).file.other
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Array}]..." \
	-command [list OpenArray mask]
    $mask(mb).file.other add separator
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Multi Ext Data Cube}]..." \
	-command [list OpenDataCubeFits mask]
    $mask(mb).file.other add separator
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Mosaic IRAF}]..." \
	-command [list OpenMosaicImageIRAFFits mask]
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
	-command [list OpenMosaicIRAFFits mask]
    $mask(mb).file.other add separator
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Mosaic WCS}]..." \
	-command [list OpenMosaicImageWCSFits mask]
    $mask(mb).file.other add command \
	-label "[msgcat::mc {Open Mosaic WCS Segment}]..." \
	-command [list OpenMosaicWCSFits mask]

    menu $mask(mb).edit
    $mask(mb).edit add command -label [msgcat::mc {Cut}] \
	-command MaskCutDialog -accelerator "${ds9(ctrl)}X"
    $mask(mb).edit add command -label [msgcat::mc {Copy}] \
	-command MaskCopyDialog -accelerator "${ds9(ctrl)}C"
    $mask(mb).edit add command -label [msgcat::mc {Paste}] \
	-command MaskPasteDialog -accelerator "${ds9(ctrl)}V"

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    frame $w.param.trans
    scale $w.param.trans.slider -from 0 -to 100 -length 300 \
	-variable mask(transparency) -orient horizontal \
	-label [msgcat::mc {Transparency}] \
	-tickinterval 20 -showvalue false -resolution 1 \
	-command MaskTransparency
    entry $w.param.trans.value -textvariable mask(transparency) -width 7
    pack $w.param.trans.slider $w.param.trans.value -side left -padx 4
    pack $w.param.trans.value

    pack $w.param.trans -side top

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command MaskApplyDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command MaskClear
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command MaskDestroyDialog
    pack $w.buttons.apply $w.buttons.clear \
	$w.buttons.close -side left -padx 10 -expand true

    UpdateMaskDialog
}

proc MaskApplyDialog {} {
    global mask

    MaskTransparency $mask(transparency)
}

proc MaskDestroyDialog {} {
    global mask

    if {[winfo exists $mask(top)]} {
	destroy $mask(top)
	destroy $mask(mb)
    }
}

proc UpdateMaskMenu {} {
    global current
    global mask
    global ds9

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateMaskMenu"
    }

    if {$current(frame) == {}} {
	return
    }

    set mask(color) [$current(frame) get mask color]
    set mask(mark) [$current(frame) get mask mark]
    set mask(transparency) [$current(frame) get mask transparency]

    switch -- [$current(frame) get type] {
	base {
	    switch -- $ds9(visual) {
		pseudocolor {
		    $ds9(mb).analysis entryconfig \
			"[msgcat::mc {Mask Parameters}]..." -state disabled
		}
		truecolor {
		    $ds9(mb).analysis entryconfig \
			"[msgcat::mc {Mask Parameters}]..." -state normal
		}
	    }
	}
	3d -
	rgb {
	    $ds9(mb).analysis entryconfig \
		"[msgcat::mc {Mask Parameters}]..." -state disabled
	}
    }
}

proc UpdateMaskDialog {} {
    global current
    global mask
    global ds9

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateMaskDialog"
    }

    if {![winfo exists $mask(top)]} {
	return
    }

    if {$current(frame) == {}} {
	return
    }

    switch -- [$current(frame) get type] {
	base {
	    switch -- $ds9(visual) {
		pseudocolor {
		    $mask(mb).file entryconfig \
			"[msgcat::mc {Open}]..." -state disabled
		    $mask(mb).file entryconfig \
			[msgcat::mc {Open Other}] -state disabled
		}
		truecolor {
		    $mask(mb).file entryconfig \
			"[msgcat::mc {Open}]..." -state normal
		    $mask(mb).file entryconfig \
			[msgcat::mc {Open Other}] -state normal

		    if {[$current(frame) has fits cube]} {
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Multi Ext Data Cube}]..." \
			    -state normal
		    } else {
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Multi Ext Data Cube}]..." \
			    -state disabled
		    }

		    if {[$current(frame) has fits mosaic]} {
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic IRAF}]..." -state normal
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
			    -state normal
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic WCS}]..." -state normal
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic WCS Segment}]..." \
			    -state normal
		    } else {
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic IRAF}]..." -state disabled
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
			    -state disabled
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic WCS}]..." -state disabled
			$mask(mb).file.other entryconfig \
			    "[msgcat::mc {Open Mosaic WCS Segment}]..." \
			    -state disabled
		    }
		}
	    }
	}
	3d -
	rgb {
	    $mask(mb).file entryconfig \
		"[msgcat::mc {Open}]..." -state disabled
	    $mask(mb).file entryconfig \
		[msgcat::mc {Open Other}] -state disabled
	}
    }
}

proc MaskLoad {} {
    global current
    global mask

    set rr [MaskParamsDialog]
    if {$current(frame) != {}} {
	if {$rr} {
	    $current(frame) mask color $mask(color)
	    $current(frame) mask mark $mask(mark)
	}
    }
    return $rr
}

proc MaskParamsDialog {} {
    global mask
    global ed

    set w {.mskd}

    set ed(ok) 0
    set ed(color) $mask(color)
    set ed(mark) $mask(mark)

    DialogCreate $w [msgcat::mc {Mask Parameters}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons  -side top -ipadx 4 -ipady 4 -fill x -expand true

    set ed(color,msg) [msgcat::mc [string totitle $ed(color)]]
    label $w.param.colortitle -text [msgcat::mc {Color}]
    menubutton $w.param.colorbutton -relief raised \
	-textvariable ed(color,msg) -menu $w.param.colorbutton.menu
    DefaultColorMenu $w.param.colorbutton.menu ed(color) \
	[list DefaultColor ed]

    grid $w.param.colortitle $w.param.colorbutton - -padx 4 -sticky w

    label $w.param.marktitle -text [msgcat::mc {Block on}]
    radiobutton $w.param.markz -text [msgcat::mc {Zero}] \
	-variable ed(mark) -value 0 
    radiobutton $w.param.marknz -text [msgcat::mc {Non-zero}] \
	-variable ed(mark) -value 1
    label $w.param.marktitle2 -text [msgcat::mc {Value}]
    grid $w.param.marktitle $w.param.markz $w.param.marknz $w.param.marktitle2 \
	-padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set mask(color) [string tolower $ed(color)]
	set mask(mark) $ed(mark)
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc ProcessMaskCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global mask
    global current

    set rr {}

    switch -- [string tolower [lindex $var $i]] {
	color {
	    incr i
	    set mask(color) [lindex $var $i]
	    if {$current(frame) != {}} {
		$current(frame) mask color $mask(color)
	    }
	}
	mark {
	    incr i
	    set mask(mark) [lindex $var $i]
	    if {$current(frame) != {}} {
		$current(frame) mask mark $mask(mark)
	    }
	}
	transparency {
	    incr i
	    set mask(transparency) [lindex $var $i]
	    if {$current(frame) != {}} {
		$current(frame) mask transparency $mask(transparency)
	    }
	    MaskTransparency $mask(transparency)
	}
	clear {
	    MaskClear
	}

	default {
	    set rr mask
	    incr i -1
	}
    }

    return $rr
}

proc ProcessSendMaskCmd {proc id param} {
    global mask

    switch -- [string tolower $param] {
	color {$proc $id "$mask(color)\n"}
	mark {$proc $id "$mask(mark)\n"}
	transparency {$proc $id "$mask(transparency)\n"}
    }
}

