#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Menus

# Default colormap names
# [msgcat::mc {Grey}]
# [msgcat::mc {Red}]
# [msgcat::mc {Green}]
# [msgcat::mc {Blue}]
# [msgcat::mc {Heat}]
# [msgcat::mc {Cool}]
# [msgcat::mc {Rainbow}]
# [msgcat::mc {Standard}]
# [msgcat::mc {Staircase}]
# [msgcat::mc {Color}]

proc ColorMenu {} {
    global colorbar
    global ds9

    menu $ds9(mb).color

    # we don't have to worry about user added cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	$ds9(mb).color add radiobutton -label [msgcat::mc $title] \
	    -variable colorbar(map) -command "ChangeColormapID $ii"
	incr colorbar(count)
	incr colorbar(cmaps)
    }

    $ds9(mb).color add separator
    $ds9(mb).color add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable colorbar(invert) -command InvertColorbar
    $ds9(mb).color add command -label [msgcat::mc {Reset Colormap}] \
	-command ResetColormap
    $ds9(mb).color add separator
    $ds9(mb).color add command -label "[msgcat::mc {Colormap Parameters}]..." \
	-command ColormapDialog
}

proc PrefsDialogColor {} {
    global prefs
    global colorbar

    set w $prefs(tabs)

    frame $w.color
    $prefs(tabs) tab configure color -window $w.color -fill both

    frame $w.color.menu -relief ridge -bd 1
    frame $w.color.colorbar -relief ridge -bd 1
    pack $w.color.menu $w.color.colorbar \
	-side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.color

    # Default Menu
    set f $w.color.menu.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Default}]
    menubutton $f.menu -relief raised -text [msgcat::mc {Menu}] \
	-menu $f.menu.menu
    PrefsDialogButtonbarColor $f.buttonbar

    grid $f.title $f.menu $f.buttonbar -padx 4 -sticky w

    set m $f.menu.menu
    menu $m

    # we don't wany any user added cmaps
    set id [colorbar list id]
    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	set jj [lindex $id $ii]
	set title [colorbar get name $jj]
	$m add radiobutton -label [msgcat::mc $title] \
	    -variable pcolorbar(map)
    }
    $m add separator
    $m add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable pcolorbar(invert)

    # Colorbar
    set f $w.color.colorbar.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Colorbar}]
    radiobutton $f.horz -text {Horizontal} \
	-variable pcolorbar(orientation) -value horizontal
    radiobutton $f.vert -text {Vertical} \
	-variable pcolorbar(orientation) -value vertical

    checkbutton $f.numerics -text [msgcat::mc {Numerics}] \
	-variable pcolorbar(numerics)

    menubutton $f.font -relief raised \
	-textvariable colorbar(font) -menu $f.font.menu
    DefaultFontMenu $f.font.menu \
	pcolorbar(font) pcolorbar(font,size) pcolorbar(font,style) {}

    grid $f.title - -padx 4 -sticky w 
    grid $f.horz $f.vert -padx 4 -sticky w
    grid $f.numerics $f.font -padx 4 -sticky w
}

# Buttons

proc ButtonsColorDef {} {
    global pbuttons

    # we have a chicken or the egg problem
    # the colorbar has not been defined yet, but we must define vars 
    # before prefs are processed, so hard code all default cmaps
    array set pbuttons {
	color,grey 1
	color,red 0
	color,green 0
	color,blue 0
	color,a 1
	color,b 1
	color,bb 1
	color,he 1
	color,i8 1
	color,aips0 1
	color,sls 0
	color,hsv 0
	color,heat 1
	color,cool 1
	color,rainbow 1
	color,standard 0
	color,staircase 0
	color,color 0
	color,invert 0
	color,reset 0
	color,params 0
    }
}

proc CreateButtonsColor {} {
    global buttons
    global ds9
    global colorbar

    frame $ds9(buttons).color

    # don't worry about user created cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	set name [string tolower $title]
	RadioButton $ds9(buttons).color.$name \
	    [string tolower [msgcat::mc $title]] \
	    colorbar(map) $title "ChangeColormapID $ii"
    }

    CheckButton $ds9(buttons).color.invert \
	[string tolower [msgcat::mc {Invert}]] colorbar(invert) InvertColorbar
    ButtonButton $ds9(buttons).color.reset \
	[string tolower [msgcat::mc {Reset}]] ResetColormap
    ButtonButton $ds9(buttons).color.params \
	[string tolower [msgcat::mc {Parameters}]] ColormapDialog

    set buttons(color) {}
    # don't worry about user created cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	set name [string tolower $title]
	append buttons(color) "$ds9(buttons).color.$name pbuttons(color,$name) "
    }
    append buttons(color) "$ds9(buttons).color.invert pbuttons(color,invert) "
    append buttons(color) "$ds9(buttons).color.reset pbuttons(color,reset) "
    append buttons(color) "$ds9(buttons).color.params pbuttons(color,params) "
}

proc PrefsDialogButtonbarColor {f} {
    global buttons
    global pbuttons
    global colorbar

    menubutton $f -relief raised -text [msgcat::mc {Buttonbar}] -menu $f.menu
    
    set m $f.menu
    menu $m

    # we don't wany any user added cmaps
    set id [colorbar list id]
    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	set jj [lindex $id $ii]
	set title [colorbar get name $jj]
	set name [string tolower $title]
	$m add checkbutton -label [msgcat::mc $title] \
	    -variable pbuttons(color,$name) \
	    -command {UpdateButtons buttons(color)}
    }
    $m add separator
    $m add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable pbuttons(color,invert) \
	-command {UpdateButtons buttons(color)}
    $m add checkbutton -label [msgcat::mc {Reset Colormap}] \
	-variable pbuttons(color,reset) \
	-command {UpdateButtons buttons(color)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Colormap Parameters}]..." \
	-variable pbuttons(color,params) \
	-command {UpdateButtons buttons(color)}
}

# Support

proc UpdateColorMenu {} {
    global ds9
    global current
    global colorbar
    global buttons

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateColorMenu"
    }

    if {$current(frame) != {}} {
	switch [$current(frame) get type] {
	    base -
	    3d {
		# menus
		for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
		    $ds9(mb).color entryconfig $ii -state normal
		}

		# buttons
		# we don't wany any user added cmaps
		set id [colorbar list id]
		for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
		    set jj [lindex $id $ii]
		    set title [colorbar get name $jj]
		    set name [string tolower $title]
		    $ds9(buttons).color.$name configure -state normal
		}
	    }
	    rgb {
		# menus
		for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
		    $ds9(mb).color entryconfig $ii -state disabled
		}

		# buttons
		# we don't wany any user added cmaps
		set id [colorbar list id]
		for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
		    set jj [lindex $id $ii]
		    set title [colorbar get name $jj]
		    set name [string tolower $title]
		    $ds9(buttons).color.$name configure -state disabled
		}
	    }
	}
    } else {
	# menus
	for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
	    $ds9(mb).color entryconfig $ii -state normal
	}

	# buttons
	# we don't wany any user added cmaps
	set id [colorbar list id]
	for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	    set jj [lindex $id $ii]
	    set title [colorbar get name $jj]
	    set name [string tolower $title]
	    $ds9(buttons).color.$name configure -state normal
	}
    }

    # for aqua
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {set colorbar(map) $colorbar(map)}
    }
}

