#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc PrefsDef {} {
    global prefs
    global ds9

    set prefs(top) .pf
    set prefs(version) [lindex $ds9(version) 0]
}

proc LoadPrefs {} {
    SourceInitFile {.prf}
    FixPrefs
}

proc CheckPrefs {} {
    global ds9
    global prefs

    if {[string compare $prefs(version) [lindex $ds9(version) 0]] == -1} {
	if {[tk_messageBox -type yesno -icon question \
		 -message [msgcat::mc {DS9 has detected an older preferences file, do you wish to update?}]] \
		== {yes}} {
	    SavePrefs
	}
    }
}

proc ClearPrefs {} {
    DeleteInitFile {.prf}
    Info [msgcat::mc {Preferences have been reset to the default values. Please restart DS9 for these changes to take effect}]
}

proc SavePrefs {} {
    global ds9

    # make sure old prefs files are removed
    DeleteInitFile {.prf}

    # open prefs file
    set file [open [file join [GetEnvHome] ".$ds9(app).prf"] w]
    if {$file == {}} {
	Error [msgcat::mc {An error has occurred while saving}]
	return
    }

    puts $file "global ds9"
    puts $file "global prefs"
    puts $file "set prefs(version) [lindex $ds9(version) 0]"

    # check for wrong prefs
    puts $file "\# this is a check for to ensure a match between the"
    puts $file "\# current ds9 version matches the prefs version"
    puts $file "if {\[string compare \$prefs(version) \[lindex \$ds9(version)\ 0\]] == 1} {"
    puts $file "    tk_messageBox -type ok -icon warning -message \"\[msgcat::mc {DS9 has detected a newer version of a preferences file and therefore will not process this file.}\]\""
    puts $file "    return"
    puts $file "}"

    puts $file ""
    puts $file "\# Analysis prefs"
    global analysis
    global panalysis
    puts $file "global analysis"
    puts $file "global panalysis"
    puts $file "array set panalysis \{ [array get panalysis] \}"
    puts $file {array set analysis [array get panalysis]}

    puts $file ""
    puts $file "\# AP prefs"
    global pap
    puts $file "global pap"
    puts $file "array set pap \{ [array get pap] \}"

    puts $file ""
    puts $file "\# Bin prefs"
    global bin
    global pbin
    puts $file "global bin"
    puts $file "global pbin"
    puts $file "array set pbin \{ [array get pbin] \}"
    puts $file {array set bin [array get pbin]}

    puts $file ""
    puts $file "\# Blink prefs"
    global blink
    global pblink
    puts $file "global blink"
    puts $file "global pblink"
    puts $file "array set pblink \{ [array get pblink] \}"
    puts $file {array set blink [array get pblink]}

    puts $file ""
    puts $file "\# Buttons prefs"
    global pbuttons
    puts $file "global pbuttons"
    puts $file "array set pbuttons \{ [array get pbuttons] \}"

    puts $file ""
    puts $file "\# CAT prefs"
    global cat
    puts $file "global cat"
    puts $file "set cat(server) \{$cat(server)\}"
    puts $file "set cat(sym,shape) \{$cat(sym,shape)\}"
    puts $file "set cat(sym,color) $cat(sym,color)"
    puts $file "set cat(vot) $cat(vot)"

    puts $file ""
    puts $file "\# Colorbar prefs"
    global colorbar
    global pcolorbar
    puts $file "global colorbar"
    puts $file "global pcolorbar"
    puts $file "array set pcolorbar \{ [array get pcolorbar] \}"
    puts $file {array set colorbar [array get pcolorbar]}

    puts $file ""
    puts $file "\# Contour prefs"
    global contour
    global pcontour
    puts $file "global contour"
    puts $file "global pcontour"
    puts $file "array set pcontour \{ [array get pcontour] \}"
    puts $file {array set contour [array get pcontour]}

    puts $file ""
    puts $file "\# Coord prefs"
    global coord
    puts $file "global coord"
    puts $file "array set coord \{ [array get coord] \}"

    puts $file ""
    puts $file "\# Current prefs"
    global current
    global pcurrent
    puts $file "global current"
    puts $file "global pcurrent"
    puts $file "array set pcurrent \{ [array get pcurrent] \}"
    puts $file {array set current [array get pcurrent]}

    puts $file ""
    puts $file "\# DS9 prefs"
    global ds9
    global pds9
    puts $file "global ds9"
    puts $file "global pds9"
    puts $file "array set pds9 \{ [array get pds9] \}"
    puts $file {array set ds9 [array get pds9]}

    puts $file "set ds9(analysis,user)  \{$ds9(analysis,user)\}"
    puts $file "set ds9(analysis,user2) \{$ds9(analysis,user2)\}"
    puts $file "set ds9(analysis,user3) \{$ds9(analysis,user3)\}"
    puts $file "set ds9(analysis,user4) \{$ds9(analysis,user4)\}"
    puts $file "set ds9(automarker) $ds9(automarker)"
    puts $file "set ds9(bg,color) $ds9(bg,color)"
    puts $file "set ds9(confirm) $ds9(confirm)"
    puts $file "set ds9(dialog) $ds9(dialog)"
    puts $file "set ds9(font) $ds9(font)"
    puts $file "set ds9(font,size) $ds9(font,size)"
    puts $file "set ds9(font,style) $ds9(font,style)"
    puts $file "if \{\$ds9\(font\) \!\= \"default\"\} \{"
    puts $file "option add *font \{$ds9(font) $ds9(font,size) $ds9(font,style)\}"
    puts $file "\}"
    puts $file "set ds9(language) $ds9(language)"
    puts $file "set ds9(language,name) $ds9(language,name)"
    puts $file "set ds9(nan,color) $ds9(nan,color)"
    puts $file "set ds9(tmpdir) \{$ds9(tmpdir)\}"
    puts $file "set ds9(xpa) $ds9(xpa)"
    puts $file "set ds9(samp,auto) $ds9(samp,auto)"

    puts $file ""
    puts $file "\# Examine prefs"
    global examine
    puts $file "global examine"
    puts $file "array set examine \{ [array get examine] \}"

    puts $file ""
    puts $file "\# Graph prefs"
    global graph
    puts $file "global graph"
    puts $file "set graph(horz,grid) $graph(horz,grid)"
    puts $file "set graph(horz,log) $graph(horz,log)"
    puts $file "set graph(vert,grid) $graph(vert,grid)"
    puts $file "set graph(vert,log) $graph(vert,log)"

    puts $file ""
    puts $file "\# HTTP prefs"
    global http
    puts $file "global http"
    puts $file "array set http \{ [array get http] \}"

    puts $file ""
    puts $file "\# HV prefs"
    global hv
    puts $file "global hv"
    puts $file "array set hv \{ [array get hv] \}"

    puts $file ""
    puts $file "\# Magnifier prefs"
    global magnifier
    puts $file "global magnifier"
    puts $file "array set magnifier \{ [array get magnifier] \}"

    puts $file ""
    puts $file "\# Marker prefs"
    global marker
    global pmarker
    puts $file "global marker"
    puts $file "global pmarker"
    puts $file "array set pmarker \{ [array get pmarker] \}"
    puts $file {array set marker [array get pmarker]}

    puts $file ""
    puts $file "\# Mask prefs"
    global mask
    global pmask
    puts $file "global mask"
    puts $file "global pmask"
    puts $file "array set pmask \{ [array get pmask] \}"
    puts $file {array set mask [array get pmask]}

    puts $file ""
    puts $file "\# MinMax prefs"
    global minmax
    global pminmax
    puts $file "global minmax"
    puts $file "global pminmax"
    puts $file "array set pminmax \{ [array get pminmax] \}"
    puts $file {array set minmax [array get pminmax]}

    puts $file ""
    puts $file "\# NRES prefs"
    global nres
    puts $file "global nres"
    puts $file "set nres(server) \{$nres(server)\}"

    puts $file ""
    puts $file "\# Panner prefs"
    global panner
    puts $file "global panner"
    puts $file "array set panner \{ [array get panner] \}"

    puts $file ""
    puts $file "\# PanZoom prefs"
    global panzoom
    global ppanzoom
    puts $file "global panzoom"
    puts $file "global ppanzoom"
    puts $file "set panzoom(mode) $panzoom(mode)"
    puts $file "array set ppanzoom \{ [array get ppanzoom] \}"
    puts $file {array set panzoom [array get ppanzoom]}

    puts $file ""
    puts $file "\# Postscript prefs"
    global ps
    global pps
    puts $file "global ps"
    puts $file "global pps"
    puts $file "array set pps \{ [array get pps] \}"
    puts $file {array set ps [array get pps]}

    puts $file ""
    puts $file "\# Scale prefs"
    global scale
    global pscale
    puts $file "global scale"
    puts $file "global pscale"
    puts $file "array set pscale \{ [array get pscale] \}"
    puts $file {array set scale [array get pscale]}

    puts $file ""
    puts $file "\# Smooth prefs"
    global smooth
    global psmooth
    puts $file "global smooth"
    puts $file "global psmooth"
    puts $file "array set psmooth \{ [array get psmooth] \}"
    puts $file {array set smooth [array get psmooth]}

    puts $file ""
    puts $file "\# Tile prefs"
    global tile
    global ptile
    puts $file "global tile"
    puts $file "global ptile"
    puts $file "array set ptile \{ [array get ptile] \}"
    puts $file {array set tile [array get ptile]}

    puts $file ""
    puts $file "\# View prefs"
    global view
    global pview
    puts $file "global view"
    puts $file "global pview"
    puts $file "array set pview \{ [array get pview] \}"
    puts $file {array set view [array get pview]}

    puts $file ""
    puts $file "\# WCS prefs"
    global wcs
    global pwcs
    puts $file "global wcs"
    puts $file "global pwcs"
    puts $file "array set pwcs \{ [array get pwcs] \}"
    puts $file {array set wcs [array get pwcs]}

    puts $file ""
    puts $file "\# VO prefs"
    global vo
    puts $file "global vo"
    puts $file "set vo(server) $vo(server)"
    puts $file "set vo(hv) $vo(hv)"
    puts $file "set vo(method) $vo(method)"

    puts $file ""
    puts $file "\# ZScale prefs"
    global zscale
    global pzscale
    puts $file "global zscale"
    puts $file "global pzscale"
    puts $file "array set pzscale \{ [array get pzscale] \}"
    puts $file {array set zscale [array get pzscale]}

    # and close
    close $file
}

# Process Cmds

proc ProcessPrefsCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global prefs
    global ds9
    global wcs

    switch -- [string tolower [lindex $var $i]] {
	clear {ClearPrefs}
	bgcolor {
	    incr i
	    set ds9(bg,color) [lindex $var $i]
	    PrefsBgColor
	}
	nancolor {
	    incr i
	    set ds9(nan,color) [lindex $var $i]
	    PrefsNaNColor
	}
    }
}

proc ProcessSendPrefsCmd {proc id param} {
    global ds9

    switch -- [string tolower $param] {
	bgcolor {$proc $id "$ds9(bg,color)\n"}
	nancolor {$proc $id "$ds9(nan,color)\n"}
    }
}

# Backward Compatibility
# we only support 5.x and higher

proc FixPrefs {} {
    global prefs

    # ap
    global pap
    if [info exists prefs(ap,grid)] {
	set pap(grid) $prefs(ap,grid)
	set pap(grid,log) $prefs(ap,grid,log)

	set pap(discrete) $prefs(ap,discrete)
	set pap(discrete,symbol) $prefs(ap,discrete,symbol)
	set pap(discrete,color) $prefs(ap,discrete,color)

	set pap(linear) $prefs(ap,linear)
	set pap(linear,width) $prefs(ap,linear,width)
	set pap(linear,color) $prefs(ap,linear,color)
	set pap(linear,dash) $prefs(ap,linear,dash)

	set pap(step) $prefs(ap,step)
	set pap(step,width) $prefs(ap,step,width)
	set pap(step,color) $prefs(ap,step,color)
	set pap(step,dash) $prefs(ap,step,dash)

	set pap(quadratic) $prefs(ap,quadratic)
	set pap(quadratic,width) $prefs(ap,quadratic,width)
	set pap(quadratic,color) $prefs(ap,quadratic,color)
	set pap(quadratic,dash) $prefs(ap,quadratic,dash)

#	set pap(error) $prefs(ap,error)
	set pap(error,width) $prefs(ap,error,width)
	set pap(error,color) $prefs(ap,error,color)
	set pap(error,style) $prefs(ap,error,style)

	set pap(titleFont) $prefs(ap,titleFont)
	set pap(titleSize) $prefs(ap,titleSize)
	set pap(titleStyle) $prefs(ap,titleStyle)

	set pap(textlabFont) $prefs(ap,textlabFont)
	set pap(textlabSize) $prefs(ap,textlabSize)
	set pap(textlabStyle) $prefs(ap,textlabStyle)

	set pap(numlabFont) $prefs(ap,numlabFont)
	set pap(numlabSize) $prefs(ap,numlabSize)
	set pap(numlabStyle) $prefs(ap,numlabStyle)
    }

    # buttons
    global buttons
    global pbuttons
    if [info exists buttons(file,about)] {
	foreach nn [array names buttons] {
	    set aa [split $nn ,]
	    if {[lindex $aa 1] != {}} {
		switch [lindex $aa 0] {
		    file -
		    edit -
		    view -
		    frame -
		    bin -
		    zoom -
		    scale -
		    color -
		    region -
		    wcs -
		    help {set pbuttons($nn) $buttons($nn)}
		}
	    }
	}

	unset pbuttons(scale,99.5)
	set pbuttons(scale,995) $buttons(scale,99.5)
	unset pbuttons(scale,92.5)
	set pbuttons(scale,925) $buttons(scale,92.5)

    	unset pbuttons(zoom,1/32)
	set pbuttons(zoom,i32) $buttons(zoom,1/32)
    	unset pbuttons(zoom,1/16)
	set pbuttons(zoom,i16) $buttons(zoom,1/16)
    	unset pbuttons(zoom,1/8)
	set pbuttons(zoom,i8) $buttons(zoom,1/8)
    	unset pbuttons(zoom,1/4)
	set pbuttons(zoom,i4) $buttons(zoom,1/4)
    	unset pbuttons(zoom,1/2)
	set pbuttons(zoom,i2) $buttons(zoom,1/2)
    }

    # current
    global current
    global pcurrent
    if [info exists prefs(zoom)] {
	set pcurrent(zoom) $prefs(zoom)
	set pcurrent(orient) $prefs(orient)
	set pcurrent(rotate) $prefs(rotate)
    }

    # analysis
    global analysis
    global panalysis
    if [info exists prefs(analysis,log)] {
	set panalysis(log) $prefs(analysis,log)
    }

    # mode
    global ds9
    global pds9
    if [info exists prefs(ds9,mode)] {
	set pds9(mode) $prefs(ds9,mode)
    }

    # blink
    global blink
    global pblink
    if [info exists prefs(blink,interval)] {
	set pblink(interval) $prefs(blink,interval)
    }

    # tile
    global tile
    global ptile
    if [info exists prefs(tile,mode)] {
	set ptile(mode) $prefs(tile,mode)
    }

    # colorbar
    global colorbar
    global pcolorbar
    if [info exists prefs(colorbar,map)] {
	set pcolorbar(map) $prefs(colorbar,map)
	set pcolorbar(invert) $prefs(colorbar,invert)
    }

    # marker
    global marker
    global pmarker
    if [info exists prefs(marker,shape)] {
	set pmarker(shape) $prefs(marker,shape)
	set pmarker(color) $prefs(marker,color)
	set pmarker(width) $prefs(marker,width)
	set pmarker(fixed) $prefs(marker,fixed)
	set pmarker(edit) $prefs(marker,edit)
	set pmarker(move) $prefs(marker,move)
	set pmarker(rotate) $prefs(marker,rotate)
	set pmarker(delete) $prefs(marker,delete)
	set pmarker(include) $prefs(marker,include)
	set pmarker(source) $prefs(marker,source)
	set pmarker(font) $prefs(marker,font)
	set pmarker(font,size) $prefs(marker,font,size)
	set pmarker(font,style) $prefs(marker,font,style)
	set pmarker(format) $prefs(marker,format)
	set pmarker(strip) $prefs(marker,strip)
	set pmarker(system) $prefs(marker,system)
	set pmarker(sky) $prefs(marker,sky)
	set pmarker(skyformat) $prefs(marker,skyformat)
    }
    
    if [info exists marker(dialog,system)] {
	set pmarker(dialog,system) $marker(dialog,system) 
	set pmarker(dialog,sky) $marker(dialog,sky) 
	set pmarker(dialog,skyformat) $marker(dialog,skyformat) 
	set pmarker(dialog,dist,system) $marker(dialog,dist,system) 
	set pmarker(dialog,dist,format) $marker(dialog,dist,format) 
    }

    if [info exists marker(circle,radius)] {
	set pmarker(circle,radius) $marker(circle,radius) 

	set pmarker(annulus,inner) $marker(annulus,inner) 
	set pmarker(annulus,outer) $marker(annulus,outer) 
	set pmarker(annulus,annuli) $marker(annulus,annuli) 

	set pmarker(panda,inner) $marker(panda,inner) 
	set pmarker(panda,outer) $marker(panda,outer) 
	set pmarker(panda,annuli) $marker(panda,annuli) 
	set pmarker(panda,ang1) $marker(panda,ang1) 
	set pmarker(panda,ang2) $marker(panda,ang2) 
	set pmarker(panda,angnum) $marker(panda,angnum) 

	set pmarker(ellipse,radius1) $marker(ellipse,radius1) 
	set pmarker(ellipse,radius2) $marker(ellipse,radius2) 

	set pmarker(ellipseannulus,radius1) $marker(ellipseannulus,radius1) 
	set pmarker(ellipseannulus,radius2) $marker(ellipseannulus,radius2) 
	set pmarker(ellipseannulus,radius3) $marker(ellipseannulus,radius3) 
	set pmarker(ellipseannulus,annuli) $marker(ellipseannulus,annuli) 

	set pmarker(epanda,radius1) $marker(epanda,radius1) 
	set pmarker(epanda,radius2) $marker(epanda,radius2) 
	set pmarker(epanda,radius3) $marker(epanda,radius3) 
	set pmarker(epanda,annuli) $marker(epanda,annuli) 
	set pmarker(epanda,ang1) $marker(epanda,ang1) 
	set pmarker(epanda,ang2) $marker(epanda,ang2) 
	set pmarker(epanda,angnum) $marker(epanda,angnum) 

	set pmarker(box,radius1) $marker(box,radius1) 
	set pmarker(box,radius2) $marker(box,radius2) 

	set pmarker(boxannulus,radius1) $marker(boxannulus,radius1) 
	set pmarker(boxannulus,radius2) $marker(boxannulus,radius2) 
	set pmarker(boxannulus,radius3) $marker(boxannulus,radius3) 
	set pmarker(boxannulus,annuli) $marker(boxannulus,annuli) 

	set pmarker(bpanda,radius1) $marker(bpanda,radius1) 
	set pmarker(bpanda,radius2) $marker(bpanda,radius2) 
	set pmarker(bpanda,radius3) $marker(bpanda,radius3) 
	set pmarker(bpanda,annuli) $marker(bpanda,annuli) 
	set pmarker(bpanda,ang1) $marker(bpanda,ang1) 
	set pmarker(bpanda,ang2) $marker(bpanda,ang2) 
	set pmarker(bpanda,angnum) $marker(bpanda,angnum) 

	set pmarker(projection,thick) $marker(projection,thick) 

	set pmarker(point,size) $marker(point,size) 
    }
}
