#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc WCSDef {} {
    global wcs
    global pwcs

    set wcs(system) wcs
    set wcs(sky) fk5
    set wcs(skyformat) sexagesimal
    set wcs(align) 0
    set wcs(align,system) wcs
    set wcs(align,sky) fk5

    array set pwcs [array get wcs]
}

proc UpdateWCS {} {
    global wcs

    # infobox
    # if one wcs coord system is visiable, change it
    global view
    set cnt 0
    foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$view(info,wcs$l)} {
	    incr cnt
	}
    }
    if {$cnt == 1} {
	foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    set view(info,wcs$l) 0
	}
	set view(info,$wcs(system)) 1
	LayoutInfoPanel
    }

    # align
    set wcs(align,system) $wcs(system)
    set wcs(align,sky) $wcs(sky)
    AlignWCSFrame

    # compass
    global panner
    global ds9
    set panner(compass,wcs,system) $wcs(system)
    set panner(compass,wcs,sky) $wcs(sky)
    foreach f $ds9(frames) {
	$f panner wcs $panner(compass,wcs,system) $panner(compass,wcs,sky)
    }

    # grid
    global grid
    set grid(system) $wcs(system)
    set grid(sky) $wcs(sky)
    set grid(skyformat) $wcs(skyformat)
    GridUpdate

    # regions
    global marker
    set marker(system) $wcs(system)
    set marker(sky) $wcs(sky)
    set marker(skyformat) $wcs(skyformat)

    # panzoom dialog
    global panzoom
    set panzoom(system) $wcs(system)
    set panzoom(sky) $wcs(sky)
    set panzoom(skyformat) $wcs(skyformat)
    UpdateZoomDialog

    # print coordinates
    global coord
    if {$wcs(system) == "wcs"} {
	set coord(wcs) 1
    } else {
	set coord(wcs) 0
    }
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$wcs(system) == "wcs$l"} {
	    set coord(wcs$l) 1
	} else {
	    set coord(wcs$l) 0
	}
    }

    set coord(sky) $wcs(sky)
    set coord(skyformat) $wcs(skyformat)
}

proc AlignWCSFrame {} {
    global current
    global wcs
    global ds9

    if {$current(frame) != {}} {
	$current(frame) wcs align $wcs(align) $wcs(align,system) \
	    $wcs(align,sky)
	UpdateAlignWCSMenu $ds9(mb).zoom
    }
}

# Process Cmds

proc ProcessWCSCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global wcs
    global current

    set item [string tolower [lindex $var $i]]
    switch -- $item {
	system {
	    incr i
	    set wcs(system) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	sky {
	    incr i
	    set wcs(sky) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	format -
	skyformat {
	    incr i
	    set wcs(skyformat) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	align {
	    incr i
	    set wcs(align) [FromYesNo [lindex $var $i]]
	    AlignWCSFrame
	}
	reset {
	    $current(frame) wcs reset
	    UpdateWCSMenu
	    UpdateWCS
	}
	replace -
	append {
	    if {$sock != {}} {
		$current(frame) wcs $item $sock
	    } elseif {$fn != {}} {
		$current(frame) wcs $item $fn
	    } else {
		incr i
		if {[lindex $var $i] == "file"} {
		    incr i
		}
		$current(frame) wcs $item "\{[lindex $var $i]\}"
	    }
	    UpdateWCSMenu
	    UpdateWCS
	}
	default {
	    set wcs(system) $item
	    switch -- $wcs(system) {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    set wcs(sky) $wcs(system)
		    set wcs(system) wcs
		}
	    }
	    UpdateWCS
	}
    }
}

proc ProcessSendWCSCmd {proc id param} {
    global wcs

    switch -- [string tolower $param] {
	align {$proc $id [ToYesNo $wcs(align)]}
	system {$proc $id "$wcs(system)\n"}
	sky  {$proc $id "$wcs(sky)\n"}
	format -
	skyformat {$proc $id "$wcs(skyformat)\n"}
	default {$proc $id "$wcs(system)\n"}
    }
}

